@@ -11,11 +11,18 @@ typedef enum {
 	BACKGROUND_POLY_4,
 } poly_order;
 
+enum {
+	INTER_POLY = 0,
+	INTER_RBF = 1
+};
+
 struct background_data {
 	int nb_of_samples;
 	double tolerance;
 	int correction;
+    int interpolation_method;
 	poly_order degree;
+	double smoothing;
 	gboolean dither;
 	fits *fit;
 	sequence *seq;
@@ -29,11 +36,13 @@ void free_background_sample_list(GSList *list);
 GSList* add_background_sample(GSList *list, fits *fit, point pt);
 GSList* remove_background_sample(GSList *orig, fits *fit, point pt);
 void generate_background_samples(int nb_of_samples, double tolerance);
-gboolean remove_gradient_from_image(int correction, poly_order degree);
+gboolean remove_gradient_from_image(int correction, poly_order degree, double smoothing, gboolean use_dither, int interpolation_method);
 void apply_background_extraction_to_sequence(struct background_data *background_args);
 
 gboolean background_sample_is_valid(background_sample *sample);
 gdouble background_sample_get_size(background_sample *sample);
 point background_sample_get_position(background_sample *sample);
 
+void apply_background_cancel();
+
 #endif /* SRC_ALGOS_BACKGROUND_EXTRACTION_H_ */
