@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -28,11 +28,15 @@
 #include "core/OS_utils.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
-#include "gui/utils.h"
+#include "gui/registration_preview.h"
+#include "gui/image_interactions.h"
+#include "gui/image_display.h"
 #include "gui/callbacks.h"
 #include "core/processing.h"
 #include "core/command_list.h"
 #include "io/sequence.h"
+#include "io/single_image.h"
+#include "io/ser.h"
 
 #include "command_line_processor.h"
 
@@ -150,17 +154,26 @@ static void display_command_on_status_bar(int line, char *myline) {
 }
 
 static void clear_status_bar() {
-	if (!com.headless) {
-		GtkStatusbar *bar = GTK_STATUSBAR(lookup_widget("statusbar_script"));
-		gtk_statusbar_remove_all(bar, 0);
-		update_log_icon(FALSE);
-	}
+	GtkStatusbar *bar = GTK_STATUSBAR(lookup_widget("statusbar_script"));
+	gtk_statusbar_remove_all(bar, 0);
+	update_log_icon(FALSE);
 }
 
 static gboolean end_script(gpointer p) {
+	/* GTK+ code is ignored during scripts, this is a good place to redraw everything */
 	clear_status_bar();
 	set_GUI_CWD();
-	
+	update_MenuItem();
+	adjust_cutoff_from_updated_gfit();
+	redraw(REMAP_ALL);
+	redraw_previews();
+	update_zoom_label();
+	update_display_fwhm();
+	display_filename();
+	new_selection_zone();
+	set_GUI_misc();
+	set_GUI_compression();
+	update_spinCPU(0);
 	set_cursor_waiting(FALSE);
 	return FALSE;
 }
@@ -243,11 +256,15 @@ gpointer execute_script(gpointer p) {
 	}
 	g_object_unref(data_input);
 	g_object_unref(input_stream);
-	com.script = FALSE;
+
+	if (!com.headless) {
+		com.script = FALSE;
+		siril_add_idle(end_script, NULL);
+	}
+
 	/* Now we want to restore the saved cwd */
 	siril_change_dir(saved_cwd, NULL);
 	writeinitfile();
-	siril_add_idle(end_script, NULL);
 	if (!retval) {
 		siril_log_message(_("Script execution finished successfully.\n"));
 		gettimeofday(&t_end, NULL);
@@ -258,7 +275,9 @@ gpointer execute_script(gpointer p) {
 		set_progress_bar_data(msg, PROGRESS_DONE);
 	}
 	g_free(saved_cwd);
-	fprintf(stdout, "Script thread exiting\n");
+
+	if (com.script_thread)
+		siril_debug_print("Script thread exiting\n");
 	return GINT_TO_POINTER(retval);
 }
 
@@ -277,7 +296,7 @@ static void show_command_help_popup(GtkEntry *entry) {
 
 				token = g_strsplit_set(current->usage, " ", -1);
 				GString *str = g_string_new(token[0]);
-				str = g_string_prepend(str, "<span foreground=\"red\"><b>");
+				str = g_string_prepend(str, "<span foreground=\"red\" size=\"larger\"><b>");
 				str = g_string_append(str, "</b>");
 				if (token[1] != NULL) {
 					str = g_string_append(str,
@@ -287,10 +306,12 @@ static void show_command_help_popup(GtkEntry *entry) {
 				str = g_string_append(str, _(current->definition));
 				str = g_string_append(str, "\n\n<b>");
 				str = g_string_append(str, _("Can be used in a script: "));
-				str = g_string_append(str, "<span foreground=\"red\">");
+
 				if (current->scriptable) {
+					str = g_string_append(str, "<span foreground=\"green\">");
 					str = g_string_append(str, _("YES"));
 				} else {
+					str = g_string_append(str, "<span foreground=\"red\">");
 					str = g_string_append(str, _("NO"));
 				}
 				str = g_string_append(str, "</span></b>");
@@ -335,46 +356,46 @@ static gboolean on_command_key_press_event(GtkWidget *widget, GdkEventKey *event
 	switch (keyval) {
 	case GDK_KEY_Up:
 		handled = 1;
-		if (!com.cmd_history)
+		if (!gui.cmd_history)
 			break;
-		if (com.cmd_hist_display > 0) {
-			if (com.cmd_history[com.cmd_hist_display - 1])
-				--com.cmd_hist_display;
+		if (gui.cmd_hist_display > 0) {
+			if (gui.cmd_history[gui.cmd_hist_display - 1])
+				--gui.cmd_hist_display;
 			// display previous entry
-			gtk_entry_set_text(entry, com.cmd_history[com.cmd_hist_display]);
-		} else if (com.cmd_history[com.cmd_hist_size - 1]) {
+			gtk_entry_set_text(entry, gui.cmd_history[gui.cmd_hist_display]);
+		} else if (gui.cmd_history[gui.cmd_hist_size - 1]) {
 			// ring back, display previous
-			com.cmd_hist_display = com.cmd_hist_size - 1;
-			gtk_entry_set_text(entry, com.cmd_history[com.cmd_hist_display]);
+			gui.cmd_hist_display = gui.cmd_hist_size - 1;
+			gtk_entry_set_text(entry, gui.cmd_history[gui.cmd_hist_display]);
 		}
 		entrylength = gtk_entry_get_text_length(entry);
 		gtk_editable_set_position(editable, entrylength);
 		break;
 	case GDK_KEY_Down:
 		handled = 1;
-		if (!com.cmd_history)
+		if (!gui.cmd_history)
 			break;
-		if (com.cmd_hist_display == com.cmd_hist_current)
+		if (gui.cmd_hist_display == gui.cmd_hist_current)
 			break;
-		if (com.cmd_hist_display == com.cmd_hist_size - 1) {
-			if (com.cmd_hist_current == 0) {
+		if (gui.cmd_hist_display == gui.cmd_hist_size - 1) {
+			if (gui.cmd_hist_current == 0) {
 				// ring forward, end
 				gtk_entry_set_text(entry, "");
-				com.cmd_hist_display++;
-			} else if (com.cmd_history[0]) {
+				gui.cmd_hist_display++;
+			} else if (gui.cmd_history[0]) {
 				// ring forward, display next
-				com.cmd_hist_display = 0;
-				gtk_entry_set_text(entry, com.cmd_history[0]);
+				gui.cmd_hist_display = 0;
+				gtk_entry_set_text(entry, gui.cmd_history[0]);
 			}
 		} else {
-			if (com.cmd_hist_display == com.cmd_hist_current - 1) {
+			if (gui.cmd_hist_display == gui.cmd_hist_current - 1) {
 				// end
 				gtk_entry_set_text(entry, "");
-				com.cmd_hist_display++;
-			} else if (com.cmd_history[com.cmd_hist_display + 1]) {
+				gui.cmd_hist_display++;
+			} else if (gui.cmd_history[gui.cmd_hist_display + 1]) {
 				// display next
 				gtk_entry_set_text(entry,
-						com.cmd_history[++com.cmd_hist_display]);
+						gui.cmd_history[++gui.cmd_hist_display]);
 			}
 		}
 		entrylength = gtk_entry_get_text_length(entry);
@@ -391,18 +412,15 @@ static gboolean on_command_key_press_event(GtkWidget *widget, GdkEventKey *event
 
 int processcommand(const char *line) {
 	int wordnb = 0;
-	gchar *myline;
 	GError *error = NULL;
 
 	if (line[0] == '\0' || line[0] == '\n')
 		return 0;
 	if (line[0] == '@') { // case of files
-		if (get_thread_run()) {
+		if (get_thread_run() || get_script_thread_run()) {
 			PRINT_ANOTHER_THREAD_RUNNING;
 			return 1;
 		}
-		if (com.script_thread)
-			g_thread_join(com.script_thread);
 
 		/* Switch to console tab */
 		control_window_switch_to_tab(OUTPUT_LOGS);
@@ -433,7 +451,7 @@ int processcommand(const char *line) {
 		/* Switch to console tab */
 		control_window_switch_to_tab(OUTPUT_LOGS);
 
-		myline = strdup(line);
+		gchar *myline = strdup(line);
 		int len = strlen(line);
 		parse_line(myline, len, &wordnb);
 		if (execute_command(wordnb)) {
@@ -446,7 +464,6 @@ int processcommand(const char *line) {
 		}
 		free(myline);
 	}
-	set_cursor_waiting(FALSE);
 	return 0;
 }
 
@@ -462,7 +479,7 @@ sequence *load_sequence(const char *name, char **get_filename) {
 
 	if (!is_readable_file(file) && (!altfile || !is_readable_file(altfile))) {
 		if (check_seq(FALSE)) {
-			siril_log_message(_("No sequence `%s' found.\n"), name);
+			siril_log_color_message(_("No sequence `%s' found.\n"), "red", name);
 			g_free(file);
 			g_free(altfile);
 			return NULL;
@@ -483,12 +500,14 @@ sequence *load_sequence(const char *name, char **get_filename) {
 		}
 	}
 	if (!seq)
-		siril_log_message(_("Loading sequence `%s' failed.\n"), name);
+		siril_log_color_message(_("Loading sequence `%s' failed.\n"), "red", name);
 	else {
 		if (seq_check_basic_data(seq, FALSE) == -1) {
 			free(seq);
 			seq = NULL;
 		}
+		else if (seq->type == SEQ_SER)
+			ser_display_info(seq->ser_file);
 	}
 	g_free(file);
 	g_free(altfile);
@@ -611,22 +630,22 @@ void on_GtkCommandHelper_clicked(GtkButton *button, gpointer user_data) {
  */
 
 static void history_add_line(char *line) {
-	if (!com.cmd_history) {
-		com.cmd_hist_size = CMD_HISTORY_SIZE;
-		com.cmd_history = calloc(com.cmd_hist_size, sizeof(const char*));
-		com.cmd_hist_current = 0;
-		com.cmd_hist_display = 0;
+	if (!gui.cmd_history) {
+		gui.cmd_hist_size = CMD_HISTORY_SIZE;
+		gui.cmd_history = calloc(gui.cmd_hist_size, sizeof(const char*));
+		gui.cmd_hist_current = 0;
+		gui.cmd_hist_display = 0;
 	}
-	com.cmd_history[com.cmd_hist_current] = line;
-	com.cmd_hist_current++;
+	gui.cmd_history[gui.cmd_hist_current] = line;
+	gui.cmd_hist_current++;
 	// circle at the end
-	if (com.cmd_hist_current == com.cmd_hist_size)
-		com.cmd_hist_current = 0;
-	if (com.cmd_history[com.cmd_hist_current]) {
-		free(com.cmd_history[com.cmd_hist_current]);
-		com.cmd_history[com.cmd_hist_current] = NULL;
+	if (gui.cmd_hist_current == gui.cmd_hist_size)
+		gui.cmd_hist_current = 0;
+	if (gui.cmd_history[gui.cmd_hist_current]) {
+		free(gui.cmd_history[gui.cmd_hist_current]);
+		gui.cmd_history[gui.cmd_hist_current] = NULL;
 	}
-	com.cmd_hist_display = com.cmd_hist_current;
+	gui.cmd_hist_display = gui.cmd_hist_current;
 }
 
 void on_command_activate(GtkEntry *entry, gpointer user_data) {
