@@ -1,7 +1,7 @@
 #ifndef SIRIL_H
 #define SIRIL_H
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <glib.h>
@@ -33,7 +33,7 @@
 
 /* https://stackoverflow.com/questions/1644868/define-macro-for-debug-printing-in-c */
 #define siril_debug_print(fmt, ...) \
-   do { if (DEBUG_TEST) fprintf(stdout, fmt, ##__VA_ARGS__); } while (0)
+	do { if (DEBUG_TEST) fprintf(stdout, fmt, ##__VA_ARGS__); } while (0)
 
 #define PRINT_ALLOC_ERR fprintf(stderr, "Out of memory in %s (%s:%d) - aborting\n", __func__, __FILE__, __LINE__)
 #define PRINT_ANOTHER_THREAD_RUNNING siril_log_message(_("Another task is already in progress, ignoring new request.\n"))
@@ -51,7 +51,7 @@
        __typeof__ (b) _b = (b); \
      _a < _b ? _a : _b; })
 
-#define SWAP(a,b)  { double temp = (a); (a) = (b); (b) = temp; }
+#define SWAP(a,b) { double temp = (a); (a) = (b); (b) = temp; }
 
 #define SQR(x) ((x)*(x))
 #endif
@@ -65,8 +65,10 @@
 #define UCHAR_MAX_SINGLE ((float)UCHAR_MAX)
 #define INV_USHRT_MAX_SINGLE .000015259022f	 // 1/65535
 #define INV_UCHAR_MAX_SINGLE .0039215686f	 // 1/255
+#define INV_USHRT_MAX_DOUBLE .000015259022	 // 1/65535
+#define INV_UCHAR_MAX_DOUBLE .0039215686	 // 1/255
 
-#define BYTES_IN_A_MB 1048576	// 1024
+#define BYTES_IN_A_MB 1048576	// 1024*1024
 
 #define SEQUENCE_DEFAULT_INCLUDE TRUE	// select images by default
 
@@ -80,7 +82,7 @@ typedef unsigned short WORD;	// default type for internal image data
 #define ZOOM_MAX	128
 #define ZOOM_MIN	0.03125
 #define ZOOM_IN		1.5
-#define ZOOM_OUT    1.0 / ZOOM_IN
+#define ZOOM_OUT	1.0 / ZOOM_IN
 #define ZOOM_NONE	1.0
 #define ZOOM_FIT	-1.0	// or any value < 0
 #define ZOOM_DEFAULT	ZOOM_FIT
@@ -105,15 +107,6 @@ typedef GtkWidget SirilWidget;
 #define SIRIL_EOL "\n"
 #endif
 
-/* when requesting an image redraw, it can be asked to remap its data before redrawing it.
- * REMAP_NONE	doesn't remaps the data,
- * REMAP_ONLY	remaps only the current viewport (color channel) and the mixed (RGB) image
- * REMAP_ALL	remaps all view ports, useful when all the colors come from the same file.
- */
-#define REMAP_NONE	0
-#define REMAP_ONLY	1
-#define REMAP_ALL	2
-
 typedef enum {
 	TYPEUNDEF = (1 << 1),
 	TYPEFITS = (1 << 2),
@@ -165,13 +158,12 @@ typedef enum {
 				// the loaded sequence
 
 #define MAX_STARS 200000		// maximum length of com.stars
-#define MAX_STARS_FITTED 2000   // maximum number of stars fitted for registration
+#define MAX_STARS_FITTED 2000	// maximum number of stars fitted for registration
 
-#define INDEX_MAX 100000		// maximum index for images
+#define INDEX_MAX 65535		// maximum index for images
 
 typedef struct imdata imgdata;
 typedef struct registration_data regdata;
-typedef struct layer_info_struct layer_info;
 typedef struct sequ sequence;
 typedef struct single_image single;
 typedef struct wcs_struct wcs_info;
@@ -181,6 +173,7 @@ typedef struct libraw_config libraw;
 typedef struct phot_config phot;
 typedef struct stack_config stackconf;
 typedef struct comp_config compconf;
+typedef struct guiinf guiinfo;
 typedef struct cominf cominfo;
 typedef struct image_stats imstats;
 typedef struct rectangle_struct rectangle;
@@ -188,13 +181,26 @@ typedef struct point_struct point;
 typedef struct pointf_struct pointf;
 typedef struct pointi_struct pointi;
 typedef struct historic_struct historic;
-typedef struct fwhm_struct fitted_PSF;
+typedef struct fwhm_struct psf_star;
+typedef struct tilt_struct sensor_tilt;
 typedef struct star_finder_struct star_finder_params;
 typedef struct pref_struct preferences;
 typedef struct save_config_struct save_config;
 
 /* global structures */
 
+/* same order as in the combo box 'comboExport' */
+typedef enum {
+	EXPORT_FITS,
+	EXPORT_FITSEQ,
+	EXPORT_TIFF,
+	EXPORT_SER,
+	EXPORT_AVI,
+	EXPORT_MP4,
+	EXPORT_MP4_H265,
+	EXPORT_WEBM_VP9
+} export_format;
+
 typedef enum {
 	LINEAR_DISPLAY,	
 	LOG_DISPLAY,
@@ -203,7 +209,7 @@ typedef enum {
 	ASINH_DISPLAY,
 	STF_DISPLAY,
 	HISTEQ_DISPLAY
-} display_mode;			// used in the layer_info_struct below
+} display_mode;
 #define DISPLAY_MODE_MAX HISTEQ_DISPLAY
 
 typedef enum {
@@ -257,15 +263,6 @@ typedef enum {
 #define BAYER_FILTER_MIN BAYER_FILTER_RGGB
 #define BAYER_FILTER_MAX BAYER_FILTER_GRBG
 
-struct layer_info_struct {
-	char *name;			// name of the layer (a filter name)
-	double wavelength;		// the wavelength of the filter, in nanometres
-	WORD lo, hi;			// the values of the cutoff sliders
-	//WORD min, max;		// the min and max values of the sliders
-	gboolean cut_over, cut_under;	// display values over hi or under lo as negative
-	display_mode rendering_mode;	// defaults to LINEAR_DISPLAY
-};
-
 typedef enum { SEQ_REGULAR, SEQ_SER, SEQ_FITSEQ,
 #ifdef HAVE_FFMS2
 	SEQ_AVI,
@@ -283,7 +280,7 @@ struct imdata {
 /* registration data, exists once for each image and each layer */
 struct registration_data {
 	float shiftx, shifty;	// we could have a subpixel precision, but is it needed? saved
-	fitted_PSF *fwhm_data;	// used in PSF/FWHM registration, not saved
+	psf_star *fwhm_data;	// used in PSF/FWHM registration, not saved
 	float fwhm;		// copy of fwhm->fwhmx, used as quality indicator, saved data
 	float weighted_fwhm; // used to exclude spurious images.
 	float roundness;	// fwhm->fwhmy / fwhm->fwhmx, 0 when uninit, ]0, 1] when set
@@ -301,7 +298,6 @@ struct sequ {
 	unsigned int rx;	// first image width
 	unsigned int ry;	// first image height
 	int bitpix;		// image pixel format, from fits
-	layer_info *layers;	// info about layers, may be null if nb_layers is unknown
 	int reference_image;	// reference image for registration
 	imgdata *imgparam;	// a structure for each image of the sequence
 	regdata **regparam;	// *regparam[nb_layers], may be null if nb_layers is unknown
@@ -340,7 +336,7 @@ struct sequ {
 	
 	gboolean needs_saving;	// a dirty flag for the sequence, avoid saving it too often
 
-	fitted_PSF **photometry[MAX_SEQPSF];// psf for multiple stars for all images
+	psf_star **photometry[MAX_SEQPSF];// psf for multiple stars for all images
 	int reference_star;	// reference star for apparent magnitude (index of photometry)
 	double reference_mag;	// reference magnitude for the reference star
 	double photometry_colors[MAX_SEQPSF][3]; // colors for each photometry curve
@@ -352,7 +348,6 @@ struct single_image {
 	gboolean fileexist;// flag of existing file
 	char *comment;		// comment on how the file got there (user load, result...)
 	int nb_layers;		// number of layers embedded in each image file
-	layer_info *layers;	// info about layers
 	fits *fit;		// the fits is still gfit, but a reference doesn't hurt
 };
 
@@ -361,12 +356,13 @@ struct wcs_struct {
 	double crpix[2];
 	double crval[2];
 	double cdelt[2];
-	double cd[2][2];
-	double crota[2];
+	double pc[2][2];
 	char objctra[FLEN_VALUE];
 	char objctdec[FLEN_VALUE];
 	double ra;
 	double dec;
+	gboolean pltsolvd;
+	char pltsolvd_comment[FLEN_COMMENT];
 };
 
 struct dft_struct {
@@ -399,30 +395,35 @@ struct ffit {
 	 * */
 
 	/* data obtained from the FITS file */
-	char *header;	// entire header of the FITS file. NULL for non-FITS file.
-	WORD lo;	// MIPS-LO key in FITS file, which is "Lower visualization cutoff"
-	WORD hi;	// MIPS-HI key in FITS file, which is "Upper visualization cutoff"
-	double data_max; // used to check if 32b float is in the [0, 1] range
+	char *header;		// entire header of the FITS file. NULL for non-FITS file.
+	WORD lo;		// MIPS-LO key in FITS file, "Lower visualization cutoff"
+	WORD hi;		// MIPS-HI key in FITS file, "Upper visualization cutoff"
+	double data_max;	// used to check if 32b float is in the [0, 1] range
 	float pixel_size_x, pixel_size_y;	// XPIXSZ and YPIXSZ keys
-	unsigned int binning_x, binning_y;		// XBINNING and YBINNING keys
+	unsigned int binning_x, binning_y;	// XBINNING and YBINNING keys
 	gboolean unbinned;
 	char row_order[FLEN_VALUE];
 	GDateTime *date, *date_obs;
 	double expstart, expend;
+	char filter[FLEN_VALUE];		// FILTER key
+	char image_type[FLEN_VALUE];		// IMAGETYP key
+	char object[FLEN_VALUE];		// OBJECT key
 	char instrume[FLEN_VALUE];		// INSTRUME key
 	char telescop[FLEN_VALUE];		// TELESCOP key
 	char observer[FLEN_VALUE];		// OBSERVER key
-	char bayer_pattern[FLEN_VALUE];	// BAYERPAT key Bayer Pattern if available
+	char bayer_pattern[FLEN_VALUE];		// BAYERPAT key Bayer Pattern if available
 	int bayer_xoffset, bayer_yoffset;
 	/* data obtained from FITS or RAW files */
 	double focal_length, iso_speed, exposure, aperture, ccd_temp;
-	double cvf; // Conversion factor (e-/adu)
-	int key_gain, key_offset; // Gain, Offset values read in camera headers.
+	double livetime;		// total exposure
+	guint stackcnt;			// number of stacked frame
+	double cvf;			// Conversion factor (e-/adu)
+	int key_gain, key_offset;	// Gain, Offset values read in camera headers.
 
 	/* Plate Solving data */
-	wcs_info wcsdata;	// data from the header
+	wcs_info wcsdata;		// data from the header
 #ifdef HAVE_WCSLIB
-	struct wcsprm *wcslib;	// struct of the lib
+	struct wcsprm *wcslib;		// struct of the lib
 #endif
 
 	/* data used in the Fourier space */
@@ -449,7 +450,7 @@ struct ffit {
  * Don't forget to update conversion.c:initialize_libraw_settings() data when
  * modifying the glade settings */
 struct libraw_config {
-	double mul[3], bright;					// Color  & brightness adjustement mul[0] = red, mul[1] = green = 1, mul[2] = blue
+	double mul[3], bright;					// Color & brightness adjustement mul[0] = red, mul[1] = green = 1, mul[2] = blue
 	int auto_mul, use_camera_wb, use_auto_wb;		// White Balance parameters
 	int user_qual;						// Index of the Matrix interpolation set in dcraw, 0: bilinear, 1: VNG, 2: PPG, 3: AHD
 	int user_black;						// black point correction
@@ -461,6 +462,8 @@ struct phot_config {
 	double gain;	// A/D converter gain in electrons per ADU
 	double inner;	// Inner radius of the annulus used to measure local background.
 	double outer;	// Outer radius of the annulus used to measure local background.
+	double aperture; // flux aperture
+	gboolean force_radius; // force the aperture radius value
 	int minval, maxval;
 };
 
@@ -514,6 +517,8 @@ struct historic_struct {
 	char history[FLEN_VALUE];
 	int rx, ry;
 	data_type type;
+	wcs_info wcsdata;
+	double focal_length;
 };
 
 struct star_finder_struct {
@@ -522,9 +527,7 @@ struct star_finder_struct {
 	gboolean adjust;
 	double sigma;
 	double roundness;
-
-	/** internal variable **/
-	gboolean no_guess; // TRUE means no guess about resolution
+	double focal_length, pixel_size_x;
 };
 
 struct save_config_struct {
@@ -543,12 +546,14 @@ struct pref_struct {
 	/* state of window */
 	gboolean remember_windows;
 	rectangle main_w_pos;
+	gint pan_position;
+	gboolean is_extended;
 	gboolean is_maximized;
 
 	gboolean prepro_cfa;	// Use to save type of sensor for cosmetic correction in preprocessing
-	gboolean prepro_equalize_cfa;  // Use to save if flat will be equalized in preprocessing
-	gboolean fix_xtrans; // Use to fix xtrans darks and biases with the AF square
-	rectangle xtrans_af; // if no xtrans model found, use these values
+	gboolean prepro_equalize_cfa; // Use to save if flat will be equalized in preprocessing
+	gboolean fix_xtrans;	// Use to fix xtrans darks and biases with the AF square
+	rectangle xtrans_af;	// if no xtrans model found, use these values
 	rectangle xtrans_sample; // if no xtrans model found, use these values
 	gchar *prepro_bias_lib;
 	gboolean use_bias_lib;
@@ -562,79 +567,95 @@ struct pref_struct {
 	save_config save;
 	gboolean show_thumbnails; // show or don't show thumbnails in open dialog box
 	gint thumbnail_size;
-	gboolean check_update; // check update at startup
+	gboolean check_update;	// check update at startup
 	gboolean script_check_requires; // check the requires command in scripts
 
-	gint combo_theme;           // value of the combobox theme
-	gdouble font_scale;           // font scale
-	gboolean icon_symbolic;		// icon style
-	gchar *combo_lang;           // string value of the combobox lang
+	gint combo_theme;	// value of the combobox theme
+	gdouble font_scale;	// font scale
+	gboolean icon_symbolic;	// icon style
+	gchar *combo_lang;	// string value of the combobox lang
 
 	gchar *ext;		// FITS extension used in SIRIL
 
-	gchar *swap_dir;		// swap directory
+	gchar *swap_dir;	// swap directory
 	GSList *script_path;	// script path directories
 
-	gdouble focal;			// focal length saved in config file
-	gdouble pitch;			// pixel pitch saved in config file
+	gdouble focal;		// focal length saved in config file
+	gdouble pitch;		// pixel pitch saved in config file
 
-	libraw raw_set;			// the libraw settings
+	libraw raw_set;		// the libraw settings
 	struct debayer_config debayer;	// debayer settings
-	phot phot_set;          // photometry settings
-	gboolean catalog[6]; // Yet 6 catalogs
+	phot phot_set;		// photometry settings
+	gboolean catalog[7];	// Yet 6 catalogs and 1 user catalog
+	int position_compass;	// compass position
+	int wcs_formalism;	// formalism used in FITS header
 
-	stackconf stack; // stacking option
-	compconf comp; // compression option
-	gboolean rgb_aladin; // Add CTYPE3='RGB' in the FITS header
+	stackconf stack;	// stacking option
+	compconf comp;		// compression option
+	gboolean rgb_aladin;	// Add CTYPE3='RGB' in the FITS header
 
 	gboolean force_to_16bit;
 
 	gint selection_guides;	// number of elements of the grid guides (2 for a simple cross, 3 for the 3 thirds rule, etc.)
 
-	gchar *copyright;		// User copyright when saving image as TIFF
+	gchar *copyright;	// User copyright when saving image as TIFF
+
 };
 /**
- * End of preference structure. Read above.
+ * End of preference structure. Read above if modified.
  */
 
-/* The global data structure of siril, the only with gfit and the gtk builder,
- * declared in main.c */
-struct cominf {
-	/* current version of GTK, through GdkPixmap, doesn't handle gray images, so
-	 * graybufs are the same size than the rgbbuf with 3 times the same value */
-	guchar *graybuf[MAXGRAYVPORT];	// one B/W display buffer per viewport (R,G,B)
-	guchar *rgbbuf;			// one rgb display buffer
-	/* cairo image surface related data */
-	int surface_stride[MAXVPORT];	// allocated stride
-	int surface_height[MAXVPORT];	// allocated height
-	cairo_surface_t *surface[MAXVPORT];
-	gboolean buf_is_dirty[MAXVPORT];// dirtyness of each buffer (= need to redraw)
+/* The rendering of the main image is cached. As it can be much larger than the
+ * widget in which it's displayed, it can take a lot of time to transform it
+ * for rendering. Unfortunately, rendering is requested on each update of a
+ * label, so every time the pointer moves above the image.
+ * With the caching, the rendering is done once in a cache surface
+ * (disp_surface) and this surface is just displayed on subsequent calls if the
+ * image, the transform or the widget size has not changed.
+ */
+struct image_view {
+	GtkWidget *drawarea;
 
-	/* Color map */
-	color_map color;
+	guchar *buf;	// display buffer (image mapped to 3 times 8-bit)
+	int full_surface_stride;
+	int full_surface_height;
+	int view_width;	// drawing area size
+	int view_height;
 
-	GtkWidget *vport[MAXVPORT];	// one drawingarea per layer, one rgb drawingarea
+	cairo_surface_t *full_surface;
+	cairo_surface_t *disp_surface;	// the cache
+};
+
+
+/* The global data structure of siril gui */
+struct guiinf {
+	GtkBuilder *builder;		// the builder of the glade interface
+
+	/*********** rendering of gfit, the currently loaded image ***********/
+	struct image_view view[MAXVPORT];
 	int cvport;			// current viewport, index in the list vport above
-	GtkAdjustment *hadj[MAXVPORT];	// adjustments of vport scrollbars
-	GtkAdjustment *vadj[MAXVPORT];	// adjustments of vport scrollbars
-	sliders_mode sliders;		// 0: min/max, 1: MIPS-LO/HI, 2: user
+
+	cairo_matrix_t display_matrix;	// matrix used for image rendering (convert image to display coordinates)
+	cairo_matrix_t image_matrix;	// inverse of display_matrix (convert display to image coordinates)
+	double zoom_value;		// 1.0 is normal zoom, use get_zoom_val() to access it
+	point display_offset;		// image display offset
+
+	gboolean translating;		// the image is being translated
+
+	/*********** Color mapping **********/
+	WORD lo, hi;			// the values of the cutoff sliders
+	gboolean cut_over;		// display values over hi as negative
+	sliders_mode sliders;		// lo/hi, minmax, user
+	display_mode rendering_mode;	// pixel value scaling, defaults to LINEAR_DISPLAY
+
 	gboolean show_excluded;		// show excluded images in sequences
 
-	cairo_matrix_t display_matrix; // matrix used for image rendering (convert image to display coordinates)
-	cairo_matrix_t image_matrix; // inverse of display_matrix (convert display to image coordinates)
-	double zoom_value;		// 1.0 is normal zoom, use get_zoom_val() to access it
-	point display_offset; // image display offset
-	gboolean translating;		// true we are in display transating mode
-
-	preferences pref; // saved variable in preferences
-
 	/* selection rectangle for registration, FWHM, PSF */
-	gboolean drawing;			// true if the rectangle is being set (clicked motion)
-	pointi start;				// where the mouse was originally clicked to
-	pointi origin;				// where the selection was originally located
+	gboolean drawing;		// true if the rectangle is being set (clicked motion)
+	pointi start;			// where the mouse was originally clicked to
+	pointi origin;			// where the selection was originally located
 	gboolean freezeX, freezeY;	// locked axis during modification of a selection 
-	rectangle selection;		// coordinates of the selection rectangle
-	double ratio;				// enforced ratio of the selection (default is 0: none)
+	double ratio;			// enforced ratio of the selection (default is 0: none)
 
 	/* alignment preview data */
 	//guchar *preview_buf[PREVIEW_NB];
@@ -642,13 +663,6 @@ struct cominf {
 	GtkWidget *preview_area[PREVIEW_NB];
 	guchar *refimage_regbuffer;	// the graybuf[registration_layer] of the reference image
 	cairo_surface_t *refimage_surface;
-
-	gchar *wd;			// working directory, where images and sequences are
-	gchar *initfile;	// the path of the init file
-	
-	int reg_settings;		// Use to save registration method in the init file
-
-	gboolean cache_upscaled;	// keep up-scaled files for 'drizzle' (only used by developers)
 	
 	int filter;			// file extension filter for open/save dialogs
 
@@ -661,8 +675,20 @@ struct cominf {
 	int cmd_hist_current;		// current command index
 	int cmd_hist_display;		// displayed command index
 
+};
+
+/* The global data structure of siril core */
+struct cominf {
+	preferences pref;		// saved variable in preferences
+	gchar *wd;			// working directory, where images and sequences are
+	gchar *initfile;		// the path of the init file
+
+	int reg_settings;		// Use to save registration method in the init file
+
+	gboolean cache_upscaled;	// keep up-scaled files for 'drizzle' (only used by developers)
+
 	/* history of operations */
-	historic *history;			// the history of all operations
+	historic *history;		// the history of all operations
 	int hist_size;			// allocated size
 	int hist_current;		// current index
 	int hist_display;		// displayed index
@@ -673,13 +699,16 @@ struct cominf {
 	gsl_histogram *layers_hist[MAXVPORT]; // current image's histograms
 
 	star_finder_params starfinder_conf;	// star finder settings, from GUI or init file
-	fitted_PSF **stars;		// list of stars detected in the current image
+	psf_star **stars;		// list of stars detected in the current image
 	gboolean star_is_seqdata;	// the only star in stars belongs to seq, don't free it
 	int selected_star;		// current selected star in the GtkListStore
 	double magOffset;		// offset to reduce the real magnitude, single image
 	
 	GSList *grad_samples;
 	GSList *found_object;
+	gboolean show_wcs_grid;
+	psf_star *qphot;		// quick photometry result
+	sensor_tilt *tilt;		// tilt information
 
 	int max_thread;			// maximum of thread used for parallel execution
 
@@ -691,6 +720,8 @@ struct cominf {
 	gboolean script;		// scripts execution
 	gboolean stop_script;		// abort script execution
 	GThread *script_thread;		// reads a script and executes its commands
+
+	rectangle selection;		// coordinates of the selection rectangle
 };
 
 /* this structure is used to characterize the statistics of the image */
@@ -711,28 +742,8 @@ typedef struct Homo {
 	int Inliers;
 } Homography;
 
-#if 0
-/* TODO: this structure aims to allow the composition of several 1-channel images and make
- * more easy the management of RGB compositing */
-typedef struct image_layer_struct image_layer;
-struct image_layer_struct {
-	char		*layer_name;		/* the name of the layer (the color or band name) */
-	fits		*fit;			/* fits data of the layer */
-	int		naxis;			/* this image is naxis in fits */
-	guchar		*graybuf;		/* mapped image for display purposes */
-	int		stride;			/* Cairo data width */
-	cairo_surface_t	*surface;		/* Cairo image surface */
-	GtkWidget	*vport;			/* the viewport */
-	double		wavelength;		/* the wavelength associated with the channel */
-	guchar		rmap, gmap, bmap;	/* mapping to rgb colors, initialized function of the wavelength */
-	unsigned int	hi, lo;			/* same as fits_data->{hi,lo} but for display/compositing purposes */
-	char		*filename;		/* the filename of the fits_data file */
-	int		naxis;			/* axis number of the fits file filename, may not be 0 if it's an RGB fits for example */
-};
-#endif
-
 #ifndef MAIN
-extern GtkBuilder *builder;	// get widget references anywhere
+extern guiinfo gui;
 extern cominfo com;		// the main data struct
 extern fits gfit;		// currently loaded image
 #endif
