@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include "core/proto.h"
 #include "gui/utils.h"
 #include "gui/histogram.h"
+#include "algos/background_extraction.h"
 #include "filters/asinh.h"
 #include "filters/deconv.h"
 #include "filters/clahe.h"
@@ -33,7 +34,7 @@
 static const SirilDialogEntry entries[] =
 {
 		{"asinh_dialog", IMAGE_PROCESSING_DIALOG, TRUE, apply_asinh_cancel},
-		{"background_extraction_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
+		{"background_extraction_dialog", IMAGE_PROCESSING_DIALOG, TRUE, apply_background_cancel},
 		{"canon_fixbanding_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"CLAHE_dialog", IMAGE_PROCESSING_DIALOG, TRUE, apply_clahe_cancel},
 		{"composition_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
@@ -49,6 +50,7 @@ static const SirilDialogEntry entries[] =
 		{"ImagePlateSolver_Dial", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"linearmatch_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"Median_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
+		{"pixel_math_dialog", OTHER_DIALOG, FALSE, NULL},
 		{"resample_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"rgradient_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
 		{"rotation_dialog", IMAGE_PROCESSING_DIALOG, FALSE, NULL},
@@ -85,13 +87,12 @@ static gboolean check_escape(GtkWidget *widget, GdkEventKey *event,
 }
 
 void siril_open_dialog(gchar *id) {
-	int i;
 	gint x, y;
 	gboolean win_already_shown = FALSE;
-	GtkWindow *win = GTK_WINDOW(gtk_builder_get_object(builder, id));
+	GtkWindow *win = GTK_WINDOW(gtk_builder_get_object(gui.builder, id));
 
 	if (get_entry_by_id(id).type != INFORMATION_DIALOG) {
-		for (i = 0; i < G_N_ELEMENTS(entries); i++) {
+		for (int i = 0; i < G_N_ELEMENTS(entries); i++) {
 			GtkWidget *w = lookup_widget(entries[i].identifier);
 			if (gtk_widget_get_visible(w) && entries[i].type != INFORMATION_DIALOG) {
 				win_already_shown = TRUE;
@@ -113,7 +114,7 @@ void siril_open_dialog(gchar *id) {
 		gtk_window_set_position(win, GTK_WIN_POS_CENTER);
 	}
 	gtk_window_set_type_hint(win, GDK_WINDOW_TYPE_HINT_DIALOG);
-	gtk_window_set_transient_for(win, GTK_WINDOW(GTK_APPLICATION_WINDOW(lookup_widget("control_window"))));
+	gtk_window_set_transient_for(win, GTK_WINDOW(lookup_widget("control_window")));
 	gtk_window_present_with_time(win, GDK_CURRENT_TIME);
 }
 
