@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -39,6 +39,7 @@
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "algos/statistics.h"
+#include "algos/astrometry_solver.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
 #include "image_format_fits.h"
@@ -53,15 +54,19 @@ const char *fit_extension[] = {
 
 static char *MIPSHI[] = {"MIPS-HI", "CWHITE", "DATAMAX", NULL };
 static char *MIPSLO[] = {"MIPS-LO", "CBLACK", "DATAMIN", NULL };
-static char *PixSizeX[] = { "XPIXSZ", "XPIXELSZ", "PIXSIZE1", "PIXSIZEX", NULL };
-static char *PixSizeY[] = { "YPIXSZ", "YPIXELSZ", "PIXSIZE2", "PIXSIZEY", NULL };
-static char *BinX[] = { "XBINNING", "BINX", NULL };
-static char *BinY[] = { "YBINNING", "BINY", NULL };
-static char *Focal[] = { "FOCAL", "FOCALLEN", NULL };
+static char *PIXELSIZEX[] = { "XPIXSZ", "XPIXELSZ", "PIXSIZE1", "PIXSIZEX", NULL };
+static char *PIXELSIZEY[] = { "YPIXSZ", "YPIXELSZ", "PIXSIZE2", "PIXSIZEY", NULL };
+static char *BINX[] = { "XBINNING", "BINX", NULL };
+static char *BINY[] = { "YBINNING", "BINY", NULL };
+static char *FOCAL[] = { "FOCAL", "FOCALLEN", NULL };
 static char *CCD_TEMP[] = { "CCD-TEMP", "CCD_TEMP", "CCDTEMP", "TEMPERAT", NULL };
-static char *Exposure[] = { "EXPTIME", "EXPOSURE", NULL };
-static char *Cvf[] = { "CVF", "EGAIN", NULL };
-static char *OffsetLevel[] = { "OFFSET", "BLKLEVEL", NULL };  //Used for synthetic offset
+static char *EXPOSURE[] = { "EXPTIME", "EXPOSURE", NULL };
+static char *FILTER[] = {"FILTER", "FILT-1", NULL };
+static char *CVF[] = { "CVF", "EGAIN", NULL };
+static char *IMAGETYP[] = { "IMAGETYP", "FRAMETYP", NULL };
+static char *OFFSETLEVEL[] = { "OFFSET", "BLKLEVEL", NULL };  //Used for synthetic offset
+static char *NB_STACKED[] = { "STACKCNT", "NCOMBINE", NULL };
+
 static int CompressionMethods[] = { RICE_1, GZIP_1, GZIP_2, HCOMPRESS_1};
 
 #define __tryToFindKeywords(fptr, type, keywords, value) \
@@ -96,7 +101,7 @@ static void read_fits_date_obs_header(fits *fit) {
 
 void fit_get_photometry_data(fits *fit) {
 	read_fits_date_obs_header(fit);
-	__tryToFindKeywords(fit->fptr, TDOUBLE, Exposure, &fit->exposure);
+	__tryToFindKeywords(fit->fptr, TDOUBLE, EXPOSURE, &fit->exposure);
 }
 
 static int fit_stats(fits *fit, float *mini, float *maxi) {
@@ -268,11 +273,12 @@ void read_fits_header(fits *fit) {
 	 */
 	status = 0;
 	fits_read_key(fit->fptr, TSTRING, "PROGRAM", &str, NULL, &status);
+
+	status = 0;
+	fits_read_key(fit->fptr, TDOUBLE, "DATAMAX", &(fit->data_max), NULL, &status);
 	gboolean not_from_siril = g_ascii_strncasecmp(str, PACKAGE, strlen(PACKAGE));
 
 	if ((fit->bitpix == FLOAT_IMG && not_from_siril) || (fit->bitpix == DOUBLE_IMG)) {
-		status = 0;
-		fits_read_key(fit->fptr, TDOUBLE, "DATAMAX", &(fit->data_max), NULL, &status);
 		if (status == KEY_NO_EXIST) {
 			float mini, maxi;
 			fit_stats(fit, &mini, &maxi);
@@ -288,11 +294,11 @@ void read_fits_header(fits *fit) {
 	 * ************* CAMERA AND INSTRUMENT KEYWORDS ********************
 	 * ****************************************************************/
 
-	__tryToFindKeywords(fit->fptr, TFLOAT, PixSizeX, &fit->pixel_size_x);
-	__tryToFindKeywords(fit->fptr, TFLOAT, PixSizeY, &fit->pixel_size_y);
-	__tryToFindKeywords(fit->fptr, TUINT, BinX, &fit->binning_x);
+	__tryToFindKeywords(fit->fptr, TFLOAT, PIXELSIZEX, &fit->pixel_size_x);
+	__tryToFindKeywords(fit->fptr, TFLOAT, PIXELSIZEY, &fit->pixel_size_y);
+	__tryToFindKeywords(fit->fptr, TUINT, BINX, &fit->binning_x);
 	if (fit->binning_x <= 0) fit->binning_x = 1;
-	__tryToFindKeywords(fit->fptr, TUINT, BinY, &fit->binning_y);
+	__tryToFindKeywords(fit->fptr, TUINT, BINY, &fit->binning_y);
 	if (fit->binning_y <= 0) fit->binning_y = 1;
 
 	status = 0;
@@ -326,7 +332,7 @@ void read_fits_header(fits *fit) {
 	fits_read_key(fit->fptr, TSTRING, "DATE", &date, NULL, &status);
 	fit->date = FITS_date_to_date_time(date);
 
-	__tryToFindKeywords(fit->fptr, TDOUBLE, Focal, &fit->focal_length);
+	__tryToFindKeywords(fit->fptr, TDOUBLE, FOCAL, &fit->focal_length);
 	if (fit->focal_length <= 0.0) {
 		/* this keyword is seen in some professional images, FLENGTH is in m. */
 		double flength;
@@ -338,7 +344,17 @@ void read_fits_header(fits *fit) {
 	}
 
 	__tryToFindKeywords(fit->fptr, TDOUBLE, CCD_TEMP, &fit->ccd_temp);
-	__tryToFindKeywords(fit->fptr, TDOUBLE, Exposure, &fit->exposure);
+	__tryToFindKeywords(fit->fptr, TDOUBLE, EXPOSURE, &fit->exposure);
+	__tryToFindKeywords(fit->fptr, TUINT, NB_STACKED, &fit->stackcnt);
+
+	status = 0;
+	fits_read_key(fit->fptr, TDOUBLE, "LIVETIME", &(fit->livetime), NULL, &status);
+
+	__tryToFindKeywords(fit->fptr, TSTRING, FILTER, &fit->filter);
+	__tryToFindKeywords(fit->fptr, TSTRING, IMAGETYP, &fit->image_type);
+
+	status = 0;
+	fits_read_key(fit->fptr, TSTRING, "OBJECT", &(fit->object), NULL, &status);
 
 	status = 0;
 	fits_read_key(fit->fptr, TDOUBLE, "APERTURE", &(fit->aperture), NULL, &status);
@@ -346,12 +362,12 @@ void read_fits_header(fits *fit) {
 	status = 0;
 	fits_read_key(fit->fptr, TDOUBLE, "ISOSPEED", &(fit->iso_speed), NULL, &status);	// Non-standard keywords used in MaxIm DL
 
-	__tryToFindKeywords(fit->fptr, TDOUBLE, Cvf, &fit->cvf); // conversion gain in e-/ADU
+	__tryToFindKeywords(fit->fptr, TDOUBLE, CVF, &fit->cvf); // conversion gain in e-/ADU
 
 	status = 0;
 	fits_read_key(fit->fptr, TUSHORT, "GAIN", &(fit->key_gain), NULL, &status);  // Gain setting from camera
 
-    __tryToFindKeywords(fit->fptr, TUSHORT, OffsetLevel, &fit->key_offset); // Offset setting from camera
+    __tryToFindKeywords(fit->fptr, TUSHORT, OFFSETLEVEL, &fit->key_offset); // Offset setting from camera
 	/*******************************************************************
 	 * ******************* PLATE SOLVING KEYWORDS **********************
 	 * ****************************************************************/
@@ -382,18 +398,6 @@ void read_fits_header(fits *fit) {
 	status = 0;
 	fits_read_key(fit->fptr, TDOUBLE, "CRVAL2", &(fit->wcsdata.crval[1]), NULL, &status);
 
-	status = 0;
-	fits_read_key(fit->fptr, TDOUBLE, "CD1_1", &(fit->wcsdata.cd[0][0]), NULL, &status);
-
-	status = 0;
-	fits_read_key(fit->fptr, TDOUBLE, "CD1_2", &(fit->wcsdata.cd[0][1]), NULL, &status);
-
-	status = 0;
-	fits_read_key(fit->fptr, TDOUBLE, "CD2_1", &(fit->wcsdata.cd[1][0]), NULL, &status);
-
-	status = 0;
-	fits_read_key(fit->fptr, TDOUBLE, "CD2_2", &(fit->wcsdata.cd[1][1]), NULL, &status);
-
 	status = 0;
 	fits_read_key(fit->fptr, TDOUBLE, "CDELT1", &(fit->wcsdata.cdelt[0]), NULL, &status);
 
@@ -401,11 +405,44 @@ void read_fits_header(fits *fit) {
 	fits_read_key(fit->fptr, TDOUBLE, "CDELT2", &(fit->wcsdata.cdelt[1]), NULL, &status);
 
 	status = 0;
-	fits_read_key(fit->fptr, TDOUBLE, "CROTA1", &(fit->wcsdata.crota[0]), NULL, &status);
+	fits_read_key(fit->fptr, TDOUBLE, "PC1_1", &(fit->wcsdata.pc[0][0]), NULL, &status);
+
+	/* IF PC does not exist, check for CD */
+	if (status == KEY_NO_EXIST) {
+		double cd[2][2];
+		status = 0;
+		fits_read_key(fit->fptr, TDOUBLE, "CD1_1", &cd[0][0], NULL, &status);
+
+		status = 0;
+		fits_read_key(fit->fptr, TDOUBLE, "CD1_2", &cd[0][1], NULL, &status);
+
+		status = 0;
+		fits_read_key(fit->fptr, TDOUBLE, "CD2_1", &cd[1][0], NULL, &status);
+
+		status = 0;
+		fits_read_key(fit->fptr, TDOUBLE, "CD2_2", &cd[1][1], NULL, &status);
+
+		if (status == 0) {
+			/* we compute PC and cdelt, but before check that CD is not singular */
+			if ((cd[0][0] * cd[1][1] - cd[1][0] * cd[0][1]) != 0.0) {
+				wcs_cd_to_pc(cd, fit->wcsdata.pc, fit->wcsdata.cdelt);
+			}
+		}
+	} else {
+		status = 0;
+		fits_read_key(fit->fptr, TDOUBLE, "PC1_2", &(fit->wcsdata.pc[0][1]), NULL, &status);
+
+		status = 0;
+		fits_read_key(fit->fptr, TDOUBLE, "PC2_1", &(fit->wcsdata.pc[1][0]), NULL, &status);
+
+		status = 0;
+		fits_read_key(fit->fptr, TDOUBLE, "PC2_2", &(fit->wcsdata.pc[1][1]), NULL, &status);
+	}
 
 	status = 0;
-	fits_read_key(fit->fptr, TDOUBLE, "CROTA2", &(fit->wcsdata.crota[1]), NULL, &status);
+	fits_read_key(fit->fptr, TLOGICAL, "PLTSOLVD", &(fit->wcsdata.pltsolvd), fit->wcsdata.pltsolvd_comment, &status);
 
+	/* so now, fill the wcslib structure */
 	load_WCS_from_file(fit);
 
 	/*******************************************************************
@@ -426,6 +463,170 @@ void read_fits_header(fits *fit) {
 	fits_read_history(fit->fptr, &(fit->history));
 }
 
+static gboolean siril_str_has_prefix(char *card, char **key) {
+	int iter = 0;
+	do {
+		gchar *new_key = g_strdup_printf("%-8s=", key[iter]);
+		if (g_str_has_prefix(card, new_key)) {
+			g_free(new_key);
+			return TRUE;
+		}
+		g_free(new_key);
+		iter++;
+	} while ((key[iter]));
+	return FALSE;
+}
+
+/* remove leading and trailing ' and whitespaces */
+static void copy_string_key(char *to, char *from) {
+	strncpy(to, from + 1, FLEN_VALUE - 1);
+	to[strlen(to) - 1] = '\0';
+	to = g_strstrip(to);
+}
+
+int fits_parse_header_string(fits *fit, gchar *header) {
+	int retval = 1;
+	double cd[2][2] = { { 0.0 } };
+	gchar **token = g_strsplit(header, "\n", -1);
+	guint nargs = g_strv_length(token);
+
+	for (int i = 0; i < nargs; i++) {
+		int status = 0;
+		char card[FLEN_CARD] = { 0 };
+		char value[FLEN_VALUE] = { 0 };
+		char comment[FLEN_COMMENT] = { 0 };
+		int keytype;
+
+		char *tok = token[i];
+		/* in case of CR + LF, we remove \r */
+		if (tok[strlen(tok) - 1] == '\r') tok[strlen(tok) - 1] = '\0';
+		fits_parse_template(tok, card, &keytype, &status);
+		if (status) return status;
+		fits_parse_value(card, value, comment, &status);
+
+		if (g_str_has_prefix(card, "END")) {
+			retval = 0;
+			break;
+		} else if (siril_str_has_prefix(card, PIXELSIZEX)) {
+			fit->pixel_size_x = g_ascii_strtod(value, NULL);
+		} else if (siril_str_has_prefix(card, PIXELSIZEY)) {
+			fit->pixel_size_y = g_ascii_strtod(value, NULL);
+		} else if (siril_str_has_prefix(card, BINX)) {
+			fit->binning_x = g_ascii_strtoull(value, NULL, 10);
+		} else if (siril_str_has_prefix(card, BINY)) {
+			fit->binning_y = g_ascii_strtoull(value, NULL, 10);
+		} else if (g_str_has_prefix(card, "ROWORDER=")) {
+			copy_string_key(fit->row_order, value);
+		} else if (g_str_has_prefix(card, "INSTRUME=")) {
+			copy_string_key(fit->instrume, value);
+		} else if (g_str_has_prefix(card, "TELESCOP=")) {
+			copy_string_key(fit->telescop, value);
+		} else if (g_str_has_prefix(card, "OBSERVER=")) {
+			copy_string_key(fit->observer, value);
+		} else if (g_str_has_prefix(card, "BAYERPAT=")) {
+			copy_string_key(fit->bayer_pattern, value);
+		} else if (g_str_has_prefix(card, "XBAYROFF=")) {
+			fit->bayer_xoffset = g_ascii_strtoull(value, NULL, 10);
+		} else if (g_str_has_prefix(card, "YBAYROFF=")) {
+			fit->bayer_yoffset = g_ascii_strtoull(value, NULL, 10);
+		} else if (g_str_has_prefix(card, "DATE    =")) {
+			char tmp[FLEN_VALUE];
+			copy_string_key(tmp, value);
+			fit->date = FITS_date_to_date_time(tmp);
+		} else if (g_str_has_prefix(card, "DATE-OBS=")) {
+			char tmp[FLEN_VALUE];
+			copy_string_key(tmp, value);
+			fit->date_obs = FITS_date_to_date_time(tmp);
+		} else if (siril_str_has_prefix(card, FOCAL)) {
+			fit->focal_length = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "FLENGTH =")) {
+			fit->focal_length = g_ascii_strtod(value, NULL) * 1000.0;
+		} else if (siril_str_has_prefix(card, CCD_TEMP)) {
+			fit->ccd_temp = g_ascii_strtod(value, NULL);
+		} else if (siril_str_has_prefix(card, EXPOSURE)) {
+			fit->exposure = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "STACKCNT=")) {
+			fit->stackcnt = g_ascii_strtoull(value, NULL, 10);
+		} else if (g_str_has_prefix(card, "LIVETIME=")) {
+			fit->livetime = g_ascii_strtod(value, NULL);
+		} else if (siril_str_has_prefix(card, FILTER)) {
+			copy_string_key(fit->filter, value);
+		} else if (siril_str_has_prefix(card, IMAGETYP)) {
+			copy_string_key(fit->image_type, value);
+		} else if (g_str_has_prefix(card, "OBJECT  =")) {
+			copy_string_key(fit->object, value);
+		} else if (g_str_has_prefix(card, "APERTURE=")) {
+			fit->aperture = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "ISOSPEED=")) {
+			fit->iso_speed = g_ascii_strtod(value, NULL);
+		} else if (siril_str_has_prefix(card, CVF)) {
+			fit->cvf = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "GAIN    =")) {
+			fit->key_gain = g_ascii_strtoull(value, NULL, 10);
+		} else if (siril_str_has_prefix(card, OFFSETLEVEL)) {
+			fit->key_offset = g_ascii_strtoull(value, NULL, 10);
+		} else if (g_str_has_prefix(card, "EQUINOX =")) {
+			fit->wcsdata.equinox = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "OBJCTRA =")) {
+			copy_string_key(fit->wcsdata.objctra, value);
+		} else if (g_str_has_prefix(card, "RA      =")) {
+			fit->wcsdata.ra = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "OBJCTDEC=")) {
+			copy_string_key(fit->wcsdata.objctdec, value);
+		} else if (g_str_has_prefix(card, "DEC     =")) {
+			fit->wcsdata.dec = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CRPIX1  =")) {
+			fit->wcsdata.crpix[0] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CRPIX2  =")) {
+			fit->wcsdata.crpix[1] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CRVAL1  =")) {
+			fit->wcsdata.crval[0] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CRVAL2  =")) {
+			fit->wcsdata.crval[1] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CDELT1  =")) {
+			fit->wcsdata.cdelt[0] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CDELT2  =")) {
+			fit->wcsdata.cdelt[1] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "PC1_1   =")) {
+			fit->wcsdata.pc[0][0] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "PC1_2   =")) {
+			fit->wcsdata.pc[0][1] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "PC2_1   =")) {
+			fit->wcsdata.pc[1][0] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "PC2_2   =")) {
+			fit->wcsdata.pc[1][1] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CD1_1   =")) {
+			cd[0][0] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CD1_2   =")) {
+			cd[0][1] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CD2_1   =")) {
+			cd[1][0] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "CD2_2   =")) {
+			cd[1][1] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "PLTSOLVD=")) {
+			fit->wcsdata.pltsolvd = !g_strcmp0(value, "T") ? TRUE : FALSE;
+			strncpy(fit->wcsdata.pltsolvd_comment, comment, FLEN_COMMENT);
+		} else if (g_str_has_prefix(card, "DFTNORM0=")) {
+			fit->dft.norm[0] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "DFTNORM1=")) {
+			fit->dft.norm[1] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "DFTNORM2=")) {
+			fit->dft.norm[2] = g_ascii_strtod(value, NULL);
+		} else if (g_str_has_prefix(card, "DFTORD  =")) {
+			copy_string_key(fit->dft.ord, value);
+		} else if (g_str_has_prefix(card, "DFTTYPE =")) {
+			copy_string_key(fit->dft.type, value);
+		}
+	}
+	/* we compute PC and cdelt, but before check that CD is not singular */
+	if ((cd[0][0] * cd[1][1] - cd[1][0] * cd[0][1]) != 0.0) {
+		wcs_cd_to_pc(cd, fit->wcsdata.pc, fit->wcsdata.cdelt);
+	}
+	load_WCS_from_memory(fit);
+
+	return retval;
+}
+
 /* copy the header for the current HDU in a heap-allocated string */
 static int copy_header_from_hdu(fitsfile *fptr, char **header, int *strsize, int *strlength) {
 	int nkeys, status = 0;
@@ -456,6 +657,18 @@ static int copy_header_from_hdu(fitsfile *fptr, char **header, int *strsize, int
 		strcpy(*header + *strlength, "\n");
 		(*strlength)++;
 	}
+	if (*strlength + 3 + 1 >= *strsize) {
+		*strsize += 4;
+		char *newstr = realloc(*header, *strsize);
+		if (!newstr) {
+			PRINT_ALLOC_ERR;
+			free(*header);
+			return 1;
+		}
+		*header = newstr;
+	}
+	strcpy(*header + *strlength, "END");
+	(*strlength) += 3;
 	return 0;
 }
 
@@ -594,8 +807,8 @@ static void convert_data_float(int bitpix, const void *from, float *to, size_t n
 	int16_t *data16;
 	double *pixels_double;
 	long *sdata32;	// TO BE TESTED on 32-bit arch, seems to be a cfitsio bug
-	long mini = LONG_MAX;
-	long maxi = -LONG_MAX;
+	float mini = FLT_MAX;
+	float maxi = -FLT_MAX;
 	unsigned long *data32;
 	float *data32f;
 
@@ -938,6 +1151,10 @@ static void save_wcs_keywords(fits *fit) {
 		status = 0;
 		fits_update_key(fit->fptr, TSTRING, "CTYPE2", "DEC--TAN", "Coordinate type for the second axis", &status);
 		status = 0;
+		fits_update_key(fit->fptr, TSTRING, "CUNIT1", "deg","Unit of coordinates", &status);
+		status = 0;
+		fits_update_key(fit->fptr, TSTRING, "CUNIT2", "deg","Unit of coordinates", &status);
+		status = 0;
 		fits_update_key(fit->fptr, TDOUBLE, "EQUINOX", &(fit->wcsdata.equinox),	"Equatorial equinox", &status);
 	}
 	status = 0;
@@ -950,15 +1167,15 @@ static void save_wcs_keywords(fits *fit) {
 
 	if (fit->wcsdata.objctra[0] != '\0') {
 		status = 0;
-		fits_update_key(fit->fptr, TSTRING, "OBJCTRA", &(fit->wcsdata.objctra),	"Image center R.A. (hms)", &status);
+		fits_update_key(fit->fptr, TSTRING, "OBJCTRA", &(fit->wcsdata.objctra),	"Image center Right Ascension (hms)", &status);
 		status = 0;
-		fits_update_key(fit->fptr, TSTRING, "OBJCTDEC", &(fit->wcsdata.objctdec), "Image center declination (dms)", &status);
+		fits_update_key(fit->fptr, TSTRING, "OBJCTDEC", &(fit->wcsdata.objctdec), "Image center Declination (dms)", &status);
 	}
 	if (fit->wcsdata.ra > 0) {
 		status = 0;
-		fits_update_key(fit->fptr, TDOUBLE, "RA", &(fit->wcsdata.ra),	"Image center Right Ascension in degrees", &status);
+		fits_update_key(fit->fptr, TDOUBLE, "RA", &(fit->wcsdata.ra), "Image center Right Ascension (deg)", &status);
 		status = 0;
-		fits_update_key(fit->fptr, TDOUBLE, "DEC", &(fit->wcsdata.dec), "Image center Declination in degrees", &status);
+		fits_update_key(fit->fptr, TDOUBLE, "DEC", &(fit->wcsdata.dec), "Image center Declination (deg)", &status);
 	}
 	if (fit->wcsdata.crpix[0] != '\0') {
 		status = 0;
@@ -968,23 +1185,45 @@ static void save_wcs_keywords(fits *fit) {
 	}
 	if (fit->wcsdata.crval[0] != '\0') {
 		status = 0;
-		fits_update_key(fit->fptr, TDOUBLE, "CRVAL1", &(fit->wcsdata.crval[0]), "Axis1 reference value", &status);
+		fits_update_key(fit->fptr, TDOUBLE, "CRVAL1", &(fit->wcsdata.crval[0]), "Axis1 reference value (deg)", &status);
 		status = 0;
-		fits_update_key(fit->fptr, TDOUBLE, "CRVAL2", &(fit->wcsdata.crval[1]), "Axis2 reference value", &status);
+		fits_update_key(fit->fptr, TDOUBLE, "CRVAL2", &(fit->wcsdata.crval[1]), "Axis2 reference value (deg)", &status);
 	}
-	if ((fit->wcsdata.cd[0][0] != 0.0) && (fit->wcsdata.cd[0][1] != 0.0) && (fit->wcsdata.cd[1][0] != 0.0) && (fit->wcsdata.cd[1][1] != 0.0)) {
-		status = 0;
-		fits_update_key(fit->fptr, TDOUBLE, "CD1_1", &(fit->wcsdata.cd[0][0]), "Scale matrix (1, 1)", &status);
-		status = 0;
-		fits_update_key(fit->fptr, TDOUBLE, "CD1_2", &(fit->wcsdata.cd[0][1]), "Scale matrix (1, 2)", &status);
-		status = 0;
-		fits_update_key(fit->fptr, TDOUBLE, "CD2_1", &(fit->wcsdata.cd[1][0]), "Scale matrix (2, 1)", &status);
-		status = 0;
-		fits_update_key(fit->fptr, TDOUBLE, "CD2_2", &(fit->wcsdata.cd[1][1]), "Scale matrix (2, 2)", &status);
-		status = 0;
-		fits_update_key(fit->fptr, TINT, "IMAGEW", &(fit->rx), "Image width, in pixels.", &status);
-		status = 0;
-		fits_update_key(fit->fptr, TINT, "IMAGEH", &(fit->ry), "Image height, in pixels.", &status);
+
+	/* check if pc matrix exists */
+	if ((fit->wcsdata.pc[0][0] * fit->wcsdata.pc[1][1] - fit->wcsdata.pc[1][0] * fit->wcsdata.pc[0][1]) != 0.0) {
+		if (com.pref.wcs_formalism == WCS_FORMALISM_1) {
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "CDELT1", &(fit->wcsdata.cdelt[0]), "X pixel size (deg)", &status);
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "CDELT2", &(fit->wcsdata.cdelt[1]), "Y pixel size (deg)", &status);
+
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "PC1_1", &(fit->wcsdata.pc[0][0]), "Linear transformation matrix (1, 1)", &status);
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "PC1_2", &(fit->wcsdata.pc[0][1]), "Linear transformation matrix (1, 2)", &status);
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "PC2_1", &(fit->wcsdata.pc[1][0]), "Linear transformation matrix (2, 1)", &status);
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "PC2_2", &(fit->wcsdata.pc[1][1]), "Linear transformation matrix (2, 2)", &status);
+			status = 0;
+		} else {
+			double cd[2][2];
+
+			wcs_pc_to_cd(fit->wcsdata.pc, fit->wcsdata.cdelt, cd);
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "CD1_1", &cd[0][0], "Scale matrix (1, 1)", &status);
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "CD1_2", &cd[0][1], "Scale matrix (1, 2)", &status);
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "CD2_1", &cd[1][0], "Scale matrix (2, 1)", &status);
+			status = 0;
+			fits_update_key(fit->fptr, TDOUBLE, "CD2_2", &cd[1][1], "Scale matrix (2, 2)", &status);
+			status = 0;
+		}
+	}
+	if (fit->wcsdata.pltsolvd) {
+		fits_update_key(fit->fptr, TLOGICAL, "PLTSOLVD", &(fit->wcsdata.pltsolvd), fit->wcsdata.pltsolvd_comment, &status);
 	}
 }
 
@@ -1074,11 +1313,21 @@ void save_fits_header(fits *fit) {
 		g_free(formatted_date);
 	}
 
+	status = 0;
+	if (fit->stackcnt > 0)
+		fits_update_key(fit->fptr, TUINT, "STACKCNT", &(fit->stackcnt),
+				"Stack frames", &status);
+
 	status = 0;
 	if (fit->exposure > 0.)
 		fits_update_key(fit->fptr, TDOUBLE, "EXPTIME", &(fit->exposure),
 				"Exposure time [s]", &status);
 
+	status = 0;
+	if (fit->livetime > 0.)
+		fits_update_key(fit->fptr, TDOUBLE, "LIVETIME", &(fit->livetime),
+				"Exposure time after deadtime correction", &status);
+
 	status = 0;
 	if (fit->expstart > 0.)
 		fits_update_key(fit->fptr, TDOUBLE, "EXPSTART", &(fit->expstart),
@@ -1089,7 +1338,6 @@ void save_fits_header(fits *fit) {
 		fits_update_key(fit->fptr, TDOUBLE, "EXPEND", &(fit->expend),
 				"Exposure end time (standard Julian date)", &status);
 
-	/* all keywords below are non-standard */
 	status = 0;
 	if (fit->pixel_size_x > 0.)
 		fits_update_key(fit->fptr, TFLOAT, "XPIXSZ", &(fit->pixel_size_x),
@@ -1116,6 +1364,21 @@ void save_fits_header(fits *fit) {
 		fits_update_key(fit->fptr, TDOUBLE, "CCD-TEMP", &(fit->ccd_temp),
 				"CCD temp in C", &status);
 
+	status = 0;
+	if (fit->filter[0] != '\0')
+		fits_update_key(fit->fptr, TSTRING, "FILTER", &(fit->filter),
+				"Active filter name", &status);
+
+	status = 0;
+	if (fit->filter[0] != '\0')
+		fits_update_key(fit->fptr, TSTRING, "IMAGETYP", &(fit->image_type),
+				"Type of image", &status);
+
+	status = 0;
+	if (fit->object[0] != '\0')
+		fits_update_key(fit->fptr, TSTRING, "OBJECT", &(fit->object),
+				"Name of the object of interest", &status);
+
 	status = 0;
 	if (fit->aperture > 0.)
 		fits_update_key(fit->fptr, TDOUBLE, "APERTURE", &(fit->aperture),
@@ -1138,7 +1401,6 @@ void save_fits_header(fits *fit) {
 		status = 0;
 		fits_update_key(fit->fptr, TINT, "YBAYROFF", &(fit->bayer_yoffset),
 				"Y offset of Bayer array", &status);
-
 	}
 
 	status = 0;
@@ -1156,43 +1418,6 @@ void save_fits_header(fits *fit) {
 		fits_update_key(fit->fptr, TDOUBLE, "CVF", &(fit->cvf),
 				"Conversion factor (e-/adu)", &status);
 
-	/*******************************************************************
-	 * ******************* PLATE SOLVING KEYWORDS **********************
-	 * ****************************************************************/
-
-	save_wcs_keywords(fit);
-
-	/*******************************************************************
-	 * ******************** PROGRAMM KEYWORDS **************************
-	 * ****************************************************************/
-
-	status = 0;
-	char programm[32];
-	sprintf(programm, "%s v%s", PACKAGE, VERSION);
-	programm[0] = toupper(programm[0]);			// convert siril to Siril
-	fits_update_key(fit->fptr, TSTRING, "PROGRAM", programm,
-			"Software that created this HDU", &status);
-
-	/*******************************************************************
-	 * ********************* HISTORY KEYWORDS **************************
-	 * ****************************************************************/
-
-	status = 0;
-	if (fit->history) {
-		GSList *list;
-		for (list = fit->history; list; list = list->next) {
-			fits_write_history(fit->fptr, (char *)list->data, &status);
-		}
-	}
-
-	status = 0;
-	if (com.history) {
-		for (i = 0; i < com.hist_display; i++) {
-			if (com.history[i].history[0] != '\0')
-				fits_write_history(fit->fptr, com.history[i].history, &status);
-		}
-	}
-
 	/*******************************************************************
 	 * ************************* DFT KEYWORDS **************************
 	 * ****************************************************************/
@@ -1233,19 +1458,68 @@ void save_fits_header(fits *fit) {
 					comment_str, &status);
 		}
 	}
+
+	/*******************************************************************
+	 * ******************** PROGRAMM KEYWORDS **************************
+	 * ****************************************************************/
+
+	status = 0;
+	char programm[32];
+
+	sprintf(programm, "%c%s v%s", toupper(PACKAGE[0]), (char*) PACKAGE + 1, VERSION);
+	fits_update_key(fit->fptr, TSTRING, "PROGRAM", programm,
+			"Software that created this HDU", &status);
+
+	/*******************************************************************
+	 * ******************* PLATE SOLVING KEYWORDS **********************
+	 * ****************************************************************/
+
+	save_wcs_keywords(fit);
+
+	/*******************************************************************
+	 * ********************* HISTORY KEYWORDS **************************
+	 * ****************************************************************/
+
+	status = 0;
+	if (fit->history) {
+		GSList *list;
+		for (list = fit->history; list; list = list->next) {
+			fits_write_history(fit->fptr, (char *)list->data, &status);
+		}
+	}
+
+	status = 0;
+	if (com.history) {
+		for (i = 0; i < com.hist_display; i++) {
+			if (com.history[i].history[0] != '\0')
+				fits_write_history(fit->fptr, com.history[i].history, &status);
+		}
+	}
 }
 
 /********************** public functions ************************************/
 
-void get_date_data_from_fitsfile(fitsfile *fptr, GDateTime **dt, double *exposure) {
-	char date_obs[FLEN_VALUE];
+/* gets specific data from the header (for stacking).
+ * if exposure is not found, it defaults to 0
+ * if livetime is not found, exposure is assigned to it
+ * if stack_count is not found, it defaults to 1
+ */
+void get_date_data_from_fitsfile(fitsfile *fptr, GDateTime **dt, double *exposure, double *livetime, unsigned int *stack_count) {
+	*exposure = 0.0;
+	*stack_count = 1;
+	*dt = NULL;
+	
+	__tryToFindKeywords(fptr, TDOUBLE, EXPOSURE, exposure);
+	__tryToFindKeywords(fptr, TUINT, NB_STACKED, stack_count);
 
-	__tryToFindKeywords(fptr, TDOUBLE, Exposure, exposure);
 	int status = 0;
-	fits_read_key(fptr, TSTRING, "DATE-OBS", &date_obs, NULL, &status);
-	if (!status) {
+	if (fits_read_key(fptr, TDOUBLE, "LIVETIME", livetime, NULL, &status))
+		*livetime = *exposure;
+
+	char date_obs[FLEN_VALUE];
+	status = 0;
+	if (!fits_read_key(fptr, TSTRING, "DATE-OBS", &date_obs, NULL, &status))
 		*dt = FITS_date_to_date_time(date_obs);
-	}
 }
 
 int import_metadata_from_fitsfile(fitsfile *fptr, fits *to) {
@@ -1253,6 +1527,7 @@ int import_metadata_from_fitsfile(fitsfile *fptr, fits *to) {
 	from.fptr = fptr;
 	read_fits_header(&from);
 	copy_fits_metadata(&from, to);
+	clearfits(&from);
 	return 0;
 }
 
@@ -1380,7 +1655,6 @@ void clearfits(fits *fit) {
 		free(fit->stats);
 	}
 	free_wcs(fit);
-
 	memset(fit, 0, sizeof(fits));
 }
 
@@ -1773,7 +2047,7 @@ int save_opened_fits(fits *f) {
 				if (norm == USHRT_MAX_DOUBLE)
 					data8[i] = conv_to_BYTE((double)f->data[i]);
 				else
-					data8[i] = round_to_BYTE(f->data[i]);
+					data8[i] = truncate_to_BYTE(f->data[i]);
 			}
 		}
 		if (fits_write_pix(f->fptr, TBYTE, orig, pixel_count, data8, &status)) {
@@ -2051,11 +2325,12 @@ int copy_fits_metadata(fits *from, fits *to) {
 		to->date = g_date_time_ref(from->date);
 	if (from->date_obs)
 		to->date_obs = g_date_time_ref(from->date_obs);
+	strncpy(to->filter, from->filter, FLEN_VALUE);
+	strncpy(to->image_type, from->image_type, FLEN_VALUE);
+	strncpy(to->object, from->object, FLEN_VALUE);
 	strncpy(to->instrume, from->instrume, FLEN_VALUE);
 	strncpy(to->telescop, from->telescop, FLEN_VALUE);
 	strncpy(to->observer, from->observer, FLEN_VALUE);
-	strncpy(to->dft.type, from->dft.type, FLEN_VALUE);
-	strncpy(to->dft.ord, from->dft.ord, FLEN_VALUE);
 	strncpy(to->bayer_pattern, from->bayer_pattern, FLEN_VALUE);
 	strncpy(to->row_order, from->row_order, FLEN_VALUE);
 
@@ -2064,16 +2339,22 @@ int copy_fits_metadata(fits *from, fits *to) {
 	to->focal_length = from->focal_length;
 	to->iso_speed = from->iso_speed;
 	to->exposure = from->exposure;
+	to->expstart = from->expstart;
+	to->expend = from->expend;
+	to->livetime = from->livetime;
+	to->stackcnt = from->stackcnt;
 	to->aperture = from->aperture;
 	to->ccd_temp = from->ccd_temp;
 	to->cvf = from->cvf;
 	to->key_gain = from->key_gain;
 	to->key_offset = from->key_offset;	
-	to->dft.norm[0] = from->dft.norm[0];
-	to->dft.norm[1] = from->dft.norm[1];
-	to->dft.norm[2] = from->dft.norm[2];
 
+	memcpy(&to->dft, &from->dft, sizeof(dft_info));
 	memcpy(&to->wcsdata, &from->wcsdata, sizeof(wcs_info));
+#ifdef HAVE_WCSLIB
+	//wcssub()?
+#endif
+	// copy from->history?
 
 	return 0;
 }
@@ -2461,9 +2742,9 @@ void fit_debayer_buffer(fits *fit, void *newbuf) {
 }
 
 static void gray2rgb(float gray, guchar *rgb) {
-	*rgb++ = (guchar) round_to_BYTE(255. * gray);
-	*rgb++ = (guchar) round_to_BYTE(255. * gray);
-	*rgb++ = (guchar) round_to_BYTE(255. * gray);
+	*rgb++ = (guchar) roundf_to_BYTE(255.f * gray);
+	*rgb++ = (guchar) roundf_to_BYTE(255.f * gray);
+	*rgb++ = (guchar) roundf_to_BYTE(255.f * gray);
 }
 
 static GdkPixbufDestroyNotify free_preview_data(guchar *pixels, gpointer data) {
