@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -43,6 +43,7 @@
 #include "registration/matching/misc.h"
 #include "opencv/opencv.h"
 
+# define MIN_RATIO_INLIERS 30 //percentage of inliers after transformation fitting (shift, affine or homography)
 
 /* TODO:
  * check usage of openmp in functions called by these ones (to be disabled)
@@ -52,6 +53,18 @@
 static void create_output_sequence_for_global_star(struct registration_args *args);
 static void print_alignment_results(Homography H, int filenum, float FWHMx, float FWHMy, char *units);
 
+static int get_min_requires_stars(transformation_type type) {
+	switch(type) {
+	case SHIFT_TRANSFORMATION:
+	case SIMILARITY_TRANSFORMATION:
+	case AFFINE_TRANSFORMATION:
+		return 3;
+	default:
+	case HOMOGRAPHY_TRANSFORMATION:
+		return 4;
+	}
+}
+
 regdata *star_align_get_current_regdata(struct registration_args *regargs) {
 	regdata *current_regdata;
 	if (regargs->seq->regparam[regargs->layer]) {
@@ -84,40 +97,8 @@ int star_align_prepare_results(struct generic_seq_args *args) {
 			return 1;
 		}
 
-		if (args->seq->type == SEQ_SER) {
-			/* copied from seq_prepare_hook with one variation */
-			args->new_ser = malloc(sizeof(struct ser_struct));
-			if (!args->new_ser) {
-				PRINT_ALLOC_ERR;
-				return 1;
-			}
-
-			char dest[256];
-			const char *ptr = strrchr(args->seq->seqname, G_DIR_SEPARATOR);
-			if (ptr)
-				snprintf(dest, 255, "%s%s.ser", regargs->prefix, ptr + 1);
-			else
-				snprintf(dest, 255, "%s%s.ser", regargs->prefix, args->seq->seqname);
-
-			/* Here the last argument is NULL because we do not want copy SER file
-			 * from the original. Indeed in the demosaicing case this would lead to
-			 * a wrong file (B&W and not in RAW data). Moreover, header informations
-			 * (like fps, local and UTC time, ...) have no sense now since some frames
-			 * could be removed from the sequence.
-			 */
-			if (ser_create_file(dest, args->new_ser, TRUE, NULL)) {
-				free(args->new_ser);
-				args->new_ser = NULL;
-				return 1;
-			}
-
-			if (seq_prepare_writer(args))
-				return 1;
-		}
-		else if (args->seq->type == SEQ_FITSEQ) {
-			if (seq_prepare_hook(args))
-				return 1;
-		}
+		if (seq_prepare_hook(args))
+			return 1;
 	}
 
 	sadata->success = calloc(args->nb_filtered_images, sizeof(BYTE));
@@ -147,6 +128,8 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 		free(sadata->current_regdata);
 		return 1;
 	}
+	if (fit.naxes[2] == 1 && fit.bayer_pattern[0] != '\0')
+		siril_log_color_message(_("Registering a sequence opened as CFA is a bad idea.\n"), "red");
 
 	siril_log_color_message(_("Reference Image:\n"), "green");
 
@@ -160,7 +143,7 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	siril_log_message(_("Found %d stars in reference, channel #%d\n"), nb_stars, regargs->layer);
 
 
-	if (!com.stars || nb_stars < AT_MATCH_MINPAIRS) {
+	if (!com.stars || nb_stars < get_min_requires_stars(regargs->type)) {
 		siril_log_message(
 				_("There are not enough stars in reference image to perform alignment\n"));
 		args->seq->regparam[regargs->layer] = NULL;
@@ -169,7 +152,7 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 		return 1;
 	}
 	if (!com.script && &com.seq == args->seq && com.seq.current == regargs->reference_image)
-		queue_redraw(REMAP_NONE); // draw stars
+		queue_redraw(REDRAW_OVERLAY); // draw stars
 
 	sadata->ref.x = fit.rx;
 	sadata->ref.y = fit.ry;
@@ -194,19 +177,19 @@ static int star_align_prepare_hook(struct generic_seq_args *args) {
 	 * that would destroy com.stars
 	 */
 	i = 0;
-	sadata->refstars = malloc((MAX_STARS + 1) * sizeof(fitted_PSF *));
+	sadata->refstars = new_fitted_stars(MAX_STARS);
 	if (!sadata->refstars) {
 		PRINT_ALLOC_ERR;
 		return 1;
 	}
 	while (i < MAX_STARS && com.stars[i]) {
-		fitted_PSF *tmp = malloc(sizeof(fitted_PSF));
+		psf_star *tmp = new_psf_star();
 		if (!tmp) {
 			PRINT_ALLOC_ERR;
 			sadata->refstars[i] = NULL;
 			return 1;
 		}
-		memcpy(tmp, com.stars[i], sizeof(fitted_PSF));
+		memcpy(tmp, com.stars[i], sizeof(psf_star));
 		sadata->refstars[i] = tmp;
 		sadata->refstars[i+1] = NULL;
 		i++;
@@ -251,21 +234,30 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 	}
 
 	if (in_index != regargs->reference_image) {
-		fitted_PSF **stars;
+		psf_star **stars;
 		if (args->seq->type == SEQ_SER || args->seq->type == SEQ_FITSEQ) {
 			siril_log_color_message(_("Frame %d:\n"), "bold", filenum);
 		}
 
+		/* sometimes sequence are not consistent.... They shouldn't but ..... */
+		int layer;
+		if (regargs->layer > RLAYER && !isrgb(fit)) {
+			layer = RLAYER;
+			siril_log_color_message(_("It looks like your sequence contains a mix of monochrome and RGB images.\n"), "salmon");
+		} else {
+			layer = regargs->layer;
+		}
+
 		if (regargs->matchSelection && regargs->selection.w > 0 && regargs->selection.h > 0) {
-			stars = peaker(fit, regargs->layer, &com.starfinder_conf, &nb_stars, &regargs->selection, FALSE, TRUE);
+			stars = peaker(fit, layer, &com.starfinder_conf, &nb_stars, &regargs->selection, FALSE, TRUE);
 		}
 		else {
-			stars = peaker(fit, regargs->layer, &com.starfinder_conf, &nb_stars, NULL, FALSE, TRUE);
+			stars = peaker(fit, layer, &com.starfinder_conf, &nb_stars, NULL, FALSE, TRUE);
 		}
 
 		siril_log_message(_("Found %d stars in image %d, channel #%d\n"), nb_stars, filenum, regargs->layer);
 
-		if (!stars || nb_stars < AT_MATCH_MINPAIRS) {
+		if (!stars || nb_stars < get_min_requires_stars(regargs->type)) {
 			siril_log_message(
 					_("Not enough stars. Image %d skipped\n"), filenum);
 			if (stars) free_fitted_stars(stars);
@@ -286,8 +278,11 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 		double scale_min = 0.9;
 		double scale_max = 1.1;
 		retvalue = 1;
+		s_star star_list_A, star_list_B;
 		while (retvalue && attempt < NB_OF_MATCHING_TRY){
-			retvalue = new_star_match(stars, sadata->refstars, nbpoints, nobj, scale_min, scale_max, &H, FALSE);
+			retvalue = new_star_match(stars, sadata->refstars, nbpoints, nobj,
+					scale_min, scale_max, &H, FALSE, regargs->type,
+					&star_list_A, &star_list_B);
 			if (attempt == 1) {
 				scale_min = -1.0;
 				scale_max = -1.0;
@@ -302,12 +297,41 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 			free_fitted_stars(stars);
 			return 1;
 		}
-		if (H.pair_matched < regargs->min_pairs) {
+		if (H.Inliers < regargs->min_pairs) {
 			siril_log_color_message(_("Not enough star pairs (%d): Image %d skipped\n"),
-					"red", H.pair_matched, filenum);
+					"red", H.Inliers, filenum);
 			free_fitted_stars(stars);
 			return 1;
 		}
+		if (((double)H.Inliers / (double)H.pair_matched) < ((double)MIN_RATIO_INLIERS / 100.)) {
+			switch (regargs->type) {
+			case SHIFT_TRANSFORMATION:
+				siril_log_color_message(_("Less than %d%% star pairs kept by shift model, it may be too rigid for your data: Image %d skipped\n"),
+					"red", MIN_RATIO_INLIERS, filenum);
+				free_fitted_stars(stars);
+				return 1;
+			break;
+			case SIMILARITY_TRANSFORMATION:
+				siril_log_color_message(_("Less than %d%% star pairs kept by similarity model, it may be too rigid for your data: Image %d skipped\n"),
+					"red", MIN_RATIO_INLIERS, filenum);
+				free_fitted_stars(stars);
+				return 1;
+			break;
+			case AFFINE_TRANSFORMATION:
+				siril_log_color_message(_("Less than %d%% star pairs kept by affine model, it may be too rigid for your data: Image %d skipped\n"),
+					"red", MIN_RATIO_INLIERS, filenum);
+				free_fitted_stars(stars);
+				return 1;
+			break;
+			case HOMOGRAPHY_TRANSFORMATION:
+				siril_log_color_message(_("Less than %d%% star pairs kept by homography model, Image %d may show important distortion\n"),
+					"salmon", MIN_RATIO_INLIERS, filenum);
+			break;
+			default:
+			printf("Should not happen\n");
+			}
+		}
+
 
 		FWHM_average(stars, nbpoints, &FWHMx, &FWHMy, &units);
 #ifdef _OPENMP
@@ -318,11 +342,11 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 		sadata->current_regdata[in_index].roundness = FWHMy/FWHMx;
 		sadata->current_regdata[in_index].fwhm =  FWHMx;
 		sadata->current_regdata[in_index].weighted_fwhm = 2 * FWHMx
-				* (((double) sadata->fitted_stars) - (double) H.pair_matched)
+				* (((double) sadata->fitted_stars) - (double) H.Inliers)
 				/ (double) sadata->fitted_stars + FWHMx;
 
 		if (!regargs->translation_only) {
-			if (cvTransformImage(fit, H, regargs->x2upscale, regargs->interpolation)) {
+			if (cvTransformImage(fit, sadata->ref.x, sadata->ref.y, H, regargs->x2upscale, regargs->interpolation)) {
 				free_fitted_stars(stars);
 				return 1;
 			}
@@ -362,7 +386,7 @@ static int star_align_image_hook(struct generic_seq_args *args, int out_index, i
 int star_align_finalize_hook(struct generic_seq_args *args) {
 	struct star_align_data *sadata = args->user;
 	struct registration_args *regargs = sadata->regargs;
-	int i, failed = 0;
+	int failed = 0;
 
 	// images may have been excluded but selnum wasn't updated
 	fix_selnum(args->seq, FALSE);
@@ -370,7 +394,7 @@ int star_align_finalize_hook(struct generic_seq_args *args) {
 	free_fitted_stars(sadata->refstars);
 
 	if (!args->retval) {
-		for (i = 0; i < args->nb_filtered_images; i++)
+		for (int i = 0; i < args->nb_filtered_images; i++)
 			if (!sadata->success[i])
 				failed++;
 		regargs->new_total = args->nb_filtered_images - failed;
@@ -379,8 +403,7 @@ int star_align_finalize_hook(struct generic_seq_args *args) {
 			if (failed) {
 				// regargs->imgparam and regargs->regparam may have holes caused by images
 				// that failed to be registered - compact them
-				int j;
-				for (i = 0, j = 0; i < regargs->new_total; i++, j++) {
+				for (int i = 0, j = 0; i < regargs->new_total; i++, j++) {
 					while (!sadata->success[j] && j < args->nb_filtered_images) j++;
 					g_assert(sadata->success[j]);
 					if (i != j) {
@@ -589,7 +612,6 @@ static void create_output_sequence_for_global_star(struct registration_args *arg
 	seq.imgparam = args->imgparam;
 	seq.regparam = calloc(seq.nb_layers, sizeof(regdata*));
 	seq.regparam[args->layer] = args->regparam;
-	seq.layers = calloc(seq.nb_layers, sizeof(layer_info));
 	seq.beg = seq.imgparam[0].filenum;
 	seq.end = seq.imgparam[seq.number-1].filenum;
 	seq.type = args->seq->type;
@@ -608,10 +630,8 @@ static void print_alignment_results(Homography H, int filenum, float FWHMx, floa
 
 	/* Matching information */
 	siril_log_color_message(_("Matching stars in image %d: done\n"), "green", filenum);
-	gchar *str = ngettext("%d pair match.\n", "%d pair matches.\n", H.pair_matched);
-	str = g_strdup_printf(str, H.pair_matched);
-	siril_log_message(str);
-	g_free(str);
+	siril_log_message(_("Initial pair matches: %d\n"), H.pair_matched);
+	siril_log_message(_("Pair matches after fitting: %d\n"), H.Inliers);
 	inliers = 1.0 - ((((double) H.pair_matched - (double) H.Inliers)) / (double) H.pair_matched);
 	siril_log_message(_("Inliers:%*.3f\n"), 11, inliers);
 
