@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -130,7 +130,7 @@ void initialize_registration_methods() {
 
 	/* fill comboboxregmethod */
 	regcombo = GTK_COMBO_BOX_TEXT(
-			gtk_builder_get_object(builder, "comboboxregmethod"));
+			gtk_builder_get_object(gui.builder, "comboboxregmethod"));
 	gtk_combo_box_text_remove_all(regcombo);
 	i = 0;
 	while (reg_methods[i] != NULL) {
@@ -149,7 +149,7 @@ void initialize_registration_methods() {
 
 struct registration_method *get_selected_registration_method() {
 	GtkComboBoxText *regcombo = GTK_COMBO_BOX_TEXT(
-			gtk_builder_get_object(builder, "comboboxregmethod"));
+			gtk_builder_get_object(gui.builder, "comboboxregmethod"));
 	int index = 0;
 
 	gchar *text = gtk_combo_box_text_get_active_text (regcombo);
@@ -679,6 +679,26 @@ void on_comboboxregmethod_changed(GtkComboBox *box, gpointer user_data) {
 	writeinitfile();
 }
 
+void on_comboreg_transfo_changed(GtkComboBox *box, gpointer user_data) {
+	GtkAdjustment *register_minpairs = GTK_ADJUSTMENT(gtk_builder_get_object(gui.builder, "register_minpairs"));
+	double val = gtk_adjustment_get_value(register_minpairs);
+
+	switch(gtk_combo_box_get_active(box)) {
+	case SHIFT_TRANSFORMATION:
+	case SIMILARITY_TRANSFORMATION:
+	case AFFINE_TRANSFORMATION:
+		gtk_adjustment_set_lower (register_minpairs, 3);
+		break;
+	case HOMOGRAPHY_TRANSFORMATION:
+		gtk_adjustment_set_lower (register_minpairs, 4);
+		if (val < 4)
+			gtk_adjustment_set_value(register_minpairs, 4);
+		break;
+	default:
+		printf("on_comboreg_transfo_changed: Value not handled.\n");
+	}
+}
+
 /* for now, the sequence argument is used only when executing a script */
 int get_registration_layer(sequence *seq) {
 	if (!com.script) {
@@ -757,9 +777,7 @@ void update_reg_interface(gboolean dont_change_reg_radio) {
 			gtk_label_set_text(labelreginfo, _("Debayer the sequence for registration"));
 		else gtk_label_set_text(labelreginfo, "");
 		// the 3 stars method has special GUI requirements
-		if (method->method_ptr != &register_3stars) {
-			gtk_widget_set_sensitive(go_register, TRUE);
-		}
+		gtk_widget_set_sensitive(go_register, (method->method_ptr != &register_3stars));
 	} else {
 		gtk_widget_set_sensitive(go_register, FALSE);
 		if (nb_images_reg <= 1 && !selection_is_done) {
@@ -867,7 +885,7 @@ void get_the_registration_area(struct registration_args *reg_args,
 			fprintf(stdout, "final area: %d,%d,\t%dx%d\n", reg_args->selection.x,
 					reg_args->selection.y, reg_args->selection.w,
 					reg_args->selection.h);
-			redraw(com.cvport, REMAP_NONE);
+			redraw(REDRAW_OVERLAY);
 			break;
 	}
 }
@@ -880,7 +898,7 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	GtkToggleButton *regall, *follow, *matchSel, *no_translate, *x2upscale,
 			*cumul;
 	GtkComboBox *cbbt_layers;
-	GtkComboBoxText *ComboBoxRegInter;
+	GtkComboBoxText *ComboBoxRegInter, *ComboBoxTransfo;
 	GtkSpinButton *minpairs;
 
 	if (!reserve_thread()) {	// reentrant from here
@@ -921,6 +939,7 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	ComboBoxRegInter = GTK_COMBO_BOX_TEXT(lookup_widget("ComboBoxRegInter"));
 	cumul = GTK_TOGGLE_BUTTON(lookup_widget("check_button_comet"));
 	minpairs = GTK_SPIN_BUTTON(lookup_widget("spinbut_minpairs"));
+	ComboBoxTransfo = GTK_COMBO_BOX_TEXT(lookup_widget("comboreg_transfo"));
 
 	reg_args->func = method->method_ptr;
 	reg_args->seq = &com.seq;
@@ -933,6 +952,15 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	reg_args->cumul = gtk_toggle_button_get_active(cumul);
 	reg_args->prefix = gtk_entry_get_text(GTK_ENTRY(lookup_widget("regseqname_entry")));
 	reg_args->min_pairs = gtk_spin_button_get_value_as_int(minpairs);
+	reg_args->type = gtk_combo_box_get_active(GTK_COMBO_BOX(ComboBoxTransfo));
+#ifndef HAVE_CV44
+	if (reg_args->type == SHIFT_TRANSFORMATION) {
+		siril_log_color_message(_("Shift-only registration is only possible with OpenCV 4.4\n"), "red");
+		free(reg_args);
+		unreserve_thread();
+		return;
+	}
+#endif
 
 	/* We check that available disk space is enough when:
 	 * - activating the subpixel alignment, which requires generating a new
@@ -974,10 +1002,11 @@ void on_seqregister_button_clicked(GtkButton *button, gpointer user_data) {
 	reg_args->run_in_thread = TRUE;
 	reg_args->load_new_sequence = FALSE; // only TRUE for global registration. Will be updated in this case
 
+	clear_stars_list(); //to avoid problems with com.stars later on in the process
+
 	msg = siril_log_color_message(_("Registration: processing using method: %s\n"),
 			"green", method->name);
 	msg[strlen(msg) - 1] = '\0';
-	set_cursor_waiting(TRUE);
 	set_progress_bar_data(msg, PROGRESS_RESET);
 
 	start_in_reserved_thread(register_thread_func, reg_args);
@@ -1012,8 +1041,8 @@ static gboolean end_register_idle(gpointer p) {
 
 	if (!args->retval) {
 		if (!args->load_new_sequence) {
-			int chan = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("seqlist_dialog_combo")));
-			update_seqlist();
+			int chan = gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("comboboxreglayer")));
+			update_seqlist(chan);
 			fill_sequence_list(args->seq, chan, FALSE);
 			set_layers_for_registration();	// update display of available reg data
 		}
