@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2021 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -33,6 +33,7 @@
 #include "gui/progress_and_log.h"
 #include "gui/PSF_list.h"
 #include "gui/sequence_list.h"
+#include "gui/registration_preview.h"
 #include "io/image_format_fits.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
@@ -47,7 +48,8 @@
 
 static struct stacking_args stackparam = {	// parameters passed to stacking
 	NULL, NULL, -1, NULL, -1.0, 0, NULL, NULL, NULL, FALSE, { 0, 0 }, -1,
-	{ 0, 0 }, NULL, NO_REJEC, NO_NORM, { NULL, NULL, NULL}, FALSE, -1
+	{ 0, 0 }, NULL, NO_REJEC, NO_NORM, { 0 }, FALSE, FALSE, TRUE, -1,
+	FALSE, NULL, NULL, NULL, { 0 }
 };
 
 #define MAX_FILTERS 5
@@ -186,9 +188,9 @@ static void start_stacking() {
 	static GtkWidget *norm_to_max = NULL;
 
 	if (method_combo == NULL) {
-		method_combo = GTK_COMBO_BOX(gtk_builder_get_object(builder, "comboboxstack_methods"));
-		output_file = GTK_ENTRY(gtk_builder_get_object(builder, "entryresultfile"));
-		overwrite = GTK_TOGGLE_BUTTON(gtk_builder_get_object(builder, "checkbutoverwrite"));
+		method_combo = GTK_COMBO_BOX(gtk_builder_get_object(gui.builder, "comboboxstack_methods"));
+		output_file = GTK_ENTRY(gtk_builder_get_object(gui.builder, "entryresultfile"));
+		overwrite = GTK_TOGGLE_BUTTON(gtk_builder_get_object(gui.builder, "checkbutoverwrite"));
 		sigSpin[0] = GTK_SPIN_BUTTON(lookup_widget("stack_siglow_button"));
 		sigSpin[1] = GTK_SPIN_BUTTON(lookup_widget("stack_sighigh_button"));
 		rejec_combo = GTK_COMBO_BOX(lookup_widget("comborejection"));
@@ -247,7 +249,7 @@ static void start_stacking() {
 }
 
 static void _show_summary(struct stacking_args *args) {
-	const char *norm_str, *rej_str;
+	const char *norm_str;
 
 	siril_log_message(_("Integration of %d images:\n"), args->nb_images_to_stack);
 
@@ -299,6 +301,7 @@ static void _show_summary(struct stacking_args *args) {
 		siril_log_message(_("Rejection parameters ...... none\n"));
 	}
 	else {
+		const char *rej_str;
 
 		switch (args->type_of_rejection) {
 		default:
@@ -335,22 +338,6 @@ static void _show_summary(struct stacking_args *args) {
 	}
 }
 
-static void _show_bgnoise(gpointer p) {
-	if (get_thread_run()) {
-		PRINT_ANOTHER_THREAD_RUNNING;
-		return;
-	}
-	set_cursor_waiting(TRUE);
-
-	struct noise_data *args = malloc(sizeof(struct noise_data));
-	args->fit = com.uniq->fit;
-	args->verbose = FALSE;
-	args->use_idle = TRUE;
-	memset(args->bgnoise, 0.0, sizeof(double[3]));
-
-	start_in_new_thread(noise, args);
-}
-
 void clean_end_stacking(struct stacking_args *args) {
 	if (!args->retval)
 		_show_summary(args);
@@ -363,10 +350,14 @@ void clean_end_stacking(struct stacking_args *args) {
 static gboolean end_stacking(gpointer p) {
 	struct timeval t_end;
 	struct stacking_args *args = (struct stacking_args *)p;
-	fprintf(stdout, "Ending stacking idle function, retval=%d\n", args->retval);
-	stop_processing_thread();	// can it be done here in case there is no thread?
+	siril_debug_print("Ending stacking idle function, retval=%d\n", args->retval);
+	stop_processing_thread();
 
 	if (args->retval == ST_OK) {
+		/* copy result to gfit if success */
+		clearfits(&gfit);
+		memcpy(&gfit, &args->result, sizeof(fits));
+
 		clear_stars_list();
 		/* check in com.seq, because args->seq may have been replaced */
 		if (com.seq.upscale_at_stacking > 1.05)
@@ -378,18 +369,17 @@ static gboolean end_stacking(gpointer p) {
 		com.uniq = calloc(1, sizeof(single));
 		com.uniq->comment = strdup(_("Stacking result image"));
 		com.uniq->nb_layers = gfit.naxes[2];
-		com.uniq->layers = calloc(com.uniq->nb_layers, sizeof(layer_info));
 		com.uniq->fit = &gfit;
 		/* Giving summary if average rejection stacking */
 		_show_summary(args);
 		/* Giving noise estimation (new thread) */
-		_show_bgnoise(com.uniq->fit);
+		bgnoise_async(&gfit, TRUE);
 
 		/* save stacking result */
 		if (args->output_filename != NULL && args->output_filename[0] != '\0') {
-			GStatBuf st;
-			if (!g_stat(args->output_filename, &st)) {
-				int failed = !args->output_overwrite;
+			int failed = 0;
+			if (g_file_test(args->output_filename, G_FILE_TEST_EXISTS)) {
+				failed = !args->output_overwrite;
 				if (!failed) {
 					if (g_unlink(args->output_filename) == -1)
 						failed = 1;
@@ -400,30 +390,34 @@ static gboolean end_stacking(gpointer p) {
 						com.uniq->fileexist = TRUE;
 					}
 				}
-				if (failed) {
-					com.uniq->filename = strdup(_("Unsaved stacking result"));
-					com.uniq->fileexist = FALSE;
-				}
 			}
 			else {
+				gchar *dirname = g_path_get_dirname(args->output_filename);
+				if (g_mkdir_with_parents(dirname, 0755) < 0) {
+					siril_log_color_message(_("Cannot create output folder: %s\n"), "red", dirname);
+					failed = 1;
+				}
+				g_free(dirname);
 				if (!savefits(args->output_filename, &gfit)) {
 					com.uniq->filename = strdup(args->output_filename);
 					com.uniq->fileexist = TRUE;
 				} else {
-					com.uniq->filename = strdup(_("Unsaved stacking result"));
-					com.uniq->fileexist = FALSE;
+					failed = 1;
 				}
 			}
+			if (failed) {
+				com.uniq->filename = strdup(_("Unsaved stacking result"));
+				com.uniq->fileexist = FALSE;
+			}
 			display_filename();
 			set_precision_switch(); // set precision on screen
 		}
 		/* remove tmp files if exist (Drizzle) */
 		remove_tmp_drizzle_files(args);
 
-		waiting_for_thread();		// bgnoise
 		initialize_display_mode();
 
-		sliders_mode_set_state(com.sliders);
+		sliders_mode_set_state(gui.sliders);
 		set_cutoff_sliders_max_values();
 		set_sliders_value_to_gfit();
 		adjust_cutoff_from_updated_gfit();	// computes min and max
@@ -433,17 +427,20 @@ static gboolean end_stacking(gpointer p) {
 		/* update menus */
 		update_MenuItem();
 
-		redraw(com.cvport, REMAP_ALL);
+		redraw(REMAP_ALL);
 		redraw_previews();
 		sequence_list_change_current();
 		update_stack_interface(TRUE);
+		bgnoise_await();
 	} else {
+		clearfits(&args->result);
 		siril_log_color_message(_("Stacking failed, please check the log to fix your issue.\n"), "red");
 		if (args->retval == ST_ALLOC_ERROR) {
 			siril_log_message(_("It looks like there is a memory allocation error, change memory settings and try to fix it.\n"));
 		}
 	}
 
+	memset(&args->result, 0, sizeof(fits));
 	set_cursor_waiting(FALSE);
 	/* Do not display time for stack_summing_generic
 	 * cause it uses the generic function that already
@@ -465,7 +462,7 @@ void on_seqstack_button_clicked (GtkButton *button, gpointer user_data){
 void on_comboboxstack_methods_changed (GtkComboBox *box, gpointer user_data) {
 	static GtkNotebook* notebook = NULL;
 	if (!notebook)
-		notebook = GTK_NOTEBOOK(gtk_builder_get_object(builder, "notebook4"));
+		notebook = GTK_NOTEBOOK(gtk_builder_get_object(gui.builder, "notebook4"));
 	com.pref.stack.method = gtk_combo_box_get_active(box);
 
 	gtk_notebook_set_current_page(notebook, com.pref.stack.method);
@@ -758,8 +755,6 @@ static void update_filter_label() {
 	static GtkComboBox *filter_combo[] = {NULL, NULL, NULL};
 	static GtkLabel *filter_label[] = {NULL, NULL, NULL};
 	gchar *filter_str;
-	double param;
-	int filter, type;
 
 	if (!filter_combo[0]) {
 		filter_combo[0] = GTK_COMBO_BOX(lookup_widget("combofilter1"));
@@ -770,13 +765,13 @@ static void update_filter_label() {
 		filter_label[2] = GTK_LABEL(lookup_widget("labelfilter3"));
 	}
 
-	for (filter = 0; filter < 3; filter++) {
+	for (int filter = 0; filter < 3; filter++) {
 		if (!gtk_widget_get_visible(GTK_WIDGET(filter_combo[filter]))) {
 			break;
 		}
 
-		type = gtk_combo_box_get_active(filter_combo[filter]);
-		param = stackfilters[filter].param;
+		int type = gtk_combo_box_get_active(filter_combo[filter]);
+		double param = stackfilters[filter].param;
 		if (param == DBL_MIN || param == DBL_MAX || param == 0.0) {
 			if (type == ALL_IMAGES || type == SELECTED_IMAGES)
 				filter_str = g_strdup("");
@@ -896,3 +891,49 @@ static void stacking_args_deep_free(struct stacking_args *args) {
 	free(args->critical_value);
 	free(args);
 }
+
+/* verify that the parameters of the image pointed by fptr are the same as some reference values */
+int check_fits_params(fitsfile *fptr, int *oldbitpix, int *oldnaxis, long *oldnaxes) {
+	int status = 0;
+	long naxes[3] = { 0 };
+	int bitpix = 0, naxis = -1;
+	fits_get_img_param(fptr, 3, &bitpix, &naxis, naxes, &status);
+	if (status) {
+		siril_log_message(_("Opening image failed\n"));
+		fits_report_error(stderr, status); /* print error message */
+		return -1;
+	}
+	if (naxis > 3) {
+		siril_log_message(_("Stacking error: images with > 3 dimensions "
+					"are not supported\n"));
+		return -1;
+	}
+
+	if (*oldnaxis > 0) {
+		if (naxis != *oldnaxis ||
+				oldnaxes[0] != naxes[0] ||
+				oldnaxes[1] != naxes[1] ||
+				oldnaxes[2] != naxes[2]) {
+			siril_log_message(_("Stacking error: input images have "
+						"different sizes\n"));
+			return -1;
+		}
+	} else {
+		*oldnaxis = naxis;
+		oldnaxes[0] = naxes[0];
+		oldnaxes[1] = naxes[1];
+		oldnaxes[2] = naxes[2];
+	}
+
+	if (*oldbitpix > 0) {
+		if (bitpix != *oldbitpix) {
+			siril_log_message(_("Stacking error: input images have "
+						"different precision\n"));
+			return -1;
+		}
+	} else {
+		*oldbitpix = bitpix;
+	}
+
+	return 0;
+}
